"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any, Callable, Hashable, Literal, TYPE_CHECKING, TypeVar
from pandas._typing import ArrayLike, DtypeObj
from pandas.core.arrays import ExtensionArray
from pandas.core.indexes.api import Index
from pandas.core.internals.base import DataManager, SingleDataManager
from pandas import Float64Index

"""
Experimental manager based on storing a collection of 1D arrays
"""
if TYPE_CHECKING:
    ...
T = TypeVar("T", bound="BaseArrayManager")
class BaseArrayManager(DataManager):
    """
    Core internal data structure to implement DataFrame and Series.

    Alternative to the BlockManager, storing a list of 1D arrays instead of
    Blocks.

    This is *not* a public API class

    Parameters
    ----------
    arrays : Sequence of arrays
    axes : Sequence of Index
    verify_integrity : bool, default True

    """
    __slots__ = ...
    arrays: list[np.ndarray | ExtensionArray]
    _axes: list[Index]
    def __init__(self, arrays: list[np.ndarray | ExtensionArray], axes: list[Index], verify_integrity: bool = ...) -> None:
        ...
    
    def make_empty(self: T, axes=...) -> T:
        """Return an empty ArrayManager with the items axis of len 0 (no columns)"""
        ...
    
    @property
    def items(self) -> Index:
        ...
    
    @property
    def axes(self) -> list[Index]:
        """Axes is BlockManager-compatible order (columns, rows)"""
        ...
    
    @property
    def shape_proper(self) -> tuple[int, ...]:
        ...
    
    def set_axis(self, axis: int, new_labels: Index) -> None:
        ...
    
    def get_dtypes(self) -> np.ndarray:
        ...
    
    def __getstate__(self): # -> tuple[list[ndarray[Unknown, Unknown] | ExtensionArray], list[Index]]:
        ...
    
    def __setstate__(self, state) -> None:
        ...
    
    def __repr__(self) -> str:
        ...
    
    def apply(self: T, f, align_keys: list[str] | None = ..., ignore_failures: bool = ..., **kwargs) -> T:
        """
        Iterate over the arrays, collect and create a new ArrayManager.

        Parameters
        ----------
        f : str or callable
            Name of the Array method to apply.
        align_keys: List[str] or None, default None
        ignore_failures: bool, default False
        **kwargs
            Keywords to pass to `f`

        Returns
        -------
        ArrayManager
        """
        ...
    
    def apply_with_block(self: T, f, align_keys=..., swap_axis=..., **kwargs) -> T:
        ...
    
    def where(self: T, other, cond, align: bool) -> T:
        ...
    
    def setitem(self: T, indexer, value) -> T:
        ...
    
    def putmask(self: T, mask, new, align: bool = ...) -> T:
        ...
    
    def diff(self: T, n: int, axis: int) -> T:
        ...
    
    def interpolate(self: T, **kwargs) -> T:
        ...
    
    def shift(self: T, periods: int, axis: int, fill_value) -> T:
        ...
    
    def fillna(self: T, value, limit, inplace: bool, downcast) -> T:
        ...
    
    def astype(self: T, dtype, copy: bool = ..., errors: str = ...) -> T:
        ...
    
    def convert(self: T, copy: bool = ..., datetime: bool = ..., numeric: bool = ..., timedelta: bool = ...) -> T:
        ...
    
    def replace_regex(self: T, **kwargs) -> T:
        ...
    
    def replace(self: T, to_replace, value, inplace: bool) -> T:
        ...
    
    def replace_list(self: T, src_list: list[Any], dest_list: list[Any], inplace: bool = ..., regex: bool = ...) -> T:
        """do a list replace"""
        ...
    
    def to_native_types(self: T, **kwargs) -> T:
        ...
    
    @property
    def is_mixed_type(self) -> bool:
        ...
    
    @property
    def is_numeric_mixed_type(self) -> bool:
        ...
    
    @property
    def any_extension_types(self) -> bool:
        """Whether any of the blocks in this manager are extension blocks"""
        ...
    
    @property
    def is_view(self) -> bool:
        """return a boolean if we are a single block and are a view"""
        ...
    
    @property
    def is_single_block(self) -> bool:
        ...
    
    def get_bool_data(self: T, copy: bool = ...) -> T:
        """
        Select columns that are bool-dtype and object-dtype columns that are all-bool.

        Parameters
        ----------
        copy : bool, default False
            Whether to copy the blocks
        """
        ...
    
    def get_numeric_data(self: T, copy: bool = ...) -> T:
        """
        Select columns that have a numeric dtype.

        Parameters
        ----------
        copy : bool, default False
            Whether to copy the blocks
        """
        ...
    
    def copy(self: T, deep=...) -> T:
        """
        Make deep or shallow copy of ArrayManager

        Parameters
        ----------
        deep : bool or string, default True
            If False, return shallow copy (do not copy data)
            If 'all', copy data and a deep copy of the index

        Returns
        -------
        BlockManager
        """
        ...
    
    def reindex_indexer(self: T, new_axis, indexer, axis: int, fill_value=..., allow_dups: bool = ..., copy: bool = ..., only_slice: bool = ..., use_na_proxy: bool = ...) -> T:
        ...
    
    def take(self: T, indexer, axis: int = ..., verify: bool = ..., convert_indices: bool = ...) -> T:
        """
        Take items along any axis.
        """
        ...
    


class ArrayManager(BaseArrayManager):
    @property
    def ndim(self) -> Literal[2]:
        ...
    
    def __init__(self, arrays: list[np.ndarray | ExtensionArray], axes: list[Index], verify_integrity: bool = ...) -> None:
        ...
    
    def fast_xs(self, loc: int) -> SingleArrayManager:
        """
        Return the array corresponding to `frame.iloc[loc]`.

        Parameters
        ----------
        loc : int

        Returns
        -------
        np.ndarray or ExtensionArray
        """
        ...
    
    def get_slice(self, slobj: slice, axis: int = ...) -> ArrayManager:
        ...
    
    def iget(self, i: int) -> SingleArrayManager:
        """
        Return the data as a SingleArrayManager.
        """
        ...
    
    def iget_values(self, i: int) -> ArrayLike:
        """
        Return the data for column i as the values (ndarray or ExtensionArray).
        """
        ...
    
    @property
    def column_arrays(self) -> list[ArrayLike]:
        """
        Used in the JSON C code to access column arrays.
        """
        ...
    
    def iset(self, loc: int | slice | np.ndarray, value: ArrayLike, inplace: bool = ...) -> None:
        """
        Set new column(s).

        This changes the ArrayManager in-place, but replaces (an) existing
        column(s), not changing column values in-place).

        Parameters
        ----------
        loc : integer, slice or boolean mask
            Positional location (already bounds checked)
        value : np.ndarray or ExtensionArray
        inplace : bool, default False
            Whether overwrite existing array as opposed to replacing it.
        """
        ...
    
    def column_setitem(self, loc: int, idx: int | slice | np.ndarray, value) -> None:
        """
        Set values ("setitem") into a single column (not setting the full column).

        This is a method on the ArrayManager level, to avoid creating an
        intermediate Series at the DataFrame level (`s = df[loc]; s[idx] = value`)
        """
        ...
    
    def insert(self, loc: int, item: Hashable, value: ArrayLike) -> None:
        """
        Insert item at selected position.

        Parameters
        ----------
        loc : int
        item : hashable
        value : np.ndarray or ExtensionArray
        """
        ...
    
    def idelete(self, indexer) -> ArrayManager:
        """
        Delete selected locations in-place (new block and array, same BlockManager)
        """
        ...
    
    def grouped_reduce(self: T, func: Callable, ignore_failures: bool = ...) -> T:
        """
        Apply grouped reduction function columnwise, returning a new ArrayManager.

        Parameters
        ----------
        func : grouped reduction function
        ignore_failures : bool, default False
            Whether to drop columns where func raises TypeError.

        Returns
        -------
        ArrayManager
        """
        ...
    
    def reduce(self: T, func: Callable, ignore_failures: bool = ...) -> tuple[T, np.ndarray]:
        """
        Apply reduction function column-wise, returning a single-row ArrayManager.

        Parameters
        ----------
        func : reduction function
        ignore_failures : bool, default False
            Whether to drop columns where func raises TypeError.

        Returns
        -------
        ArrayManager
        np.ndarray
            Indexer of column indices that are retained.
        """
        ...
    
    def operate_blockwise(self, other: ArrayManager, array_op) -> ArrayManager:
        """
        Apply array_op blockwise with another (aligned) BlockManager.
        """
        ...
    
    def quantile(self, *, qs: Float64Index, axis: int = ..., transposed: bool = ..., interpolation=...) -> ArrayManager:
        ...
    
    def unstack(self, unstacker, fill_value) -> ArrayManager:
        """
        Return a BlockManager with all blocks unstacked.

        Parameters
        ----------
        unstacker : reshape._Unstacker
        fill_value : Any
            fill_value for newly introduced missing values.

        Returns
        -------
        unstacked : BlockManager
        """
        ...
    
    def as_array(self, dtype=..., copy: bool = ..., na_value: object = ...) -> np.ndarray:
        """
        Convert the blockmanager data into an numpy array.

        Parameters
        ----------
        dtype : object, default None
            Data type of the return array.
        copy : bool, default False
            If True then guarantee that a copy is returned. A value of
            False does not guarantee that the underlying data is not
            copied.
        na_value : object, default lib.no_default
            Value to be used as the missing value sentinel.

        Returns
        -------
        arr : ndarray
        """
        ...
    


class SingleArrayManager(BaseArrayManager, SingleDataManager):
    __slots__ = ...
    arrays: list[np.ndarray | ExtensionArray]
    _axes: list[Index]
    @property
    def ndim(self) -> Literal[1]:
        ...
    
    def __init__(self, arrays: list[np.ndarray | ExtensionArray], axes: list[Index], verify_integrity: bool = ...) -> None:
        ...
    
    def make_empty(self, axes=...) -> SingleArrayManager:
        """Return an empty ArrayManager with index/array of length 0"""
        ...
    
    @classmethod
    def from_array(cls, array, index) -> SingleArrayManager:
        ...
    
    @property
    def axes(self): # -> list[Index]:
        ...
    
    @property
    def index(self) -> Index:
        ...
    
    @property
    def dtype(self): # -> ExtensionDtype:
        ...
    
    def external_values(self): # -> ArrayLike:
        """The array that Series.values returns"""
        ...
    
    def internal_values(self): # -> ArrayLike:
        """The array that Series._values returns"""
        ...
    
    def array_values(self): # -> PandasArray | ExtensionArray:
        """The array that Series.array returns"""
        ...
    
    @property
    def is_single_block(self) -> bool:
        ...
    
    def fast_xs(self, loc: int) -> SingleArrayManager:
        ...
    
    def get_slice(self, slobj: slice, axis: int = ...) -> SingleArrayManager:
        ...
    
    def getitem_mgr(self, indexer) -> SingleArrayManager:
        ...
    
    def apply(self, func, **kwargs): # -> Self@SingleArrayManager:
        ...
    
    def setitem(self, indexer, value) -> SingleArrayManager:
        """
        Set values with indexer.

        For SingleArrayManager, this backs s[indexer] = value

        See `setitem_inplace` for a version that works inplace and doesn't
        return a new Manager.
        """
        ...
    
    def idelete(self, indexer) -> SingleArrayManager:
        """
        Delete selected locations in-place (new array, same ArrayManager)
        """
        ...
    
    def set_values(self, values: ArrayLike) -> None:
        """
        Set (replace) the values of the SingleArrayManager in place.

        Use at your own risk! This does not check if the passed values are
        valid for the current SingleArrayManager (length, dtype, etc).
        """
        ...
    
    def to_2d_mgr(self, columns: Index) -> ArrayManager:
        """
        Manager analogue of Series.to_frame
        """
        ...
    


class NullArrayProxy:
    """
    Proxy object for an all-NA array.

    Only stores the length of the array, and not the dtype. The dtype
    will only be known when actually concatenating (after determining the
    common dtype, for which this proxy is ignored).
    Using this object avoids that the internals/concat.py needs to determine
    the proper dtype and array type.
    """
    ndim = ...
    def __init__(self, n: int) -> None:
        ...
    
    @property
    def shape(self) -> tuple[int]:
        ...
    
    def to_array(self, dtype: DtypeObj) -> ArrayLike:
        """
        Helper function to create the actual all-NA array from the NullArrayProxy
        object.

        Parameters
        ----------
        arr : NullArrayProxy
        dtype : the dtype for the resulting array

        Returns
        -------
        np.ndarray or ExtensionArray
        """
        ...
    





"""
This type stub file was generated by pyright.
"""

import numpy as np
import numpy.ma as ma
from typing import Any, Sequence, TYPE_CHECKING, overload
from pandas._typing import AnyArrayLike, ArrayLike, Dtype, DtypeObj, T
from pandas import ExtensionArray, Index, Series

"""
Constructor functions intended to be shared by pd.array, Series.__init__,
and Index.__new__.

These should not depend on core.internals.
"""
if TYPE_CHECKING:
    ...
def array(data: Sequence[object] | AnyArrayLike, dtype: Dtype | None = ..., copy: bool = ...) -> ExtensionArray:
    """
    Create an array.

    Parameters
    ----------
    data : Sequence of objects
        The scalars inside `data` should be instances of the
        scalar type for `dtype`. It's expected that `data`
        represents a 1-dimensional array of data.

        When `data` is an Index or Series, the underlying array
        will be extracted from `data`.

    dtype : str, np.dtype, or ExtensionDtype, optional
        The dtype to use for the array. This may be a NumPy
        dtype or an extension type registered with pandas using
        :meth:`pandas.api.extensions.register_extension_dtype`.

        If not specified, there are two possibilities:

        1. When `data` is a :class:`Series`, :class:`Index`, or
           :class:`ExtensionArray`, the `dtype` will be taken
           from the data.
        2. Otherwise, pandas will attempt to infer the `dtype`
           from the data.

        Note that when `data` is a NumPy array, ``data.dtype`` is
        *not* used for inferring the array type. This is because
        NumPy cannot represent all the types of data that can be
        held in extension arrays.

        Currently, pandas will infer an extension dtype for sequences of

        ============================== =======================================
        Scalar Type                    Array Type
        ============================== =======================================
        :class:`pandas.Interval`       :class:`pandas.arrays.IntervalArray`
        :class:`pandas.Period`         :class:`pandas.arrays.PeriodArray`
        :class:`datetime.datetime`     :class:`pandas.arrays.DatetimeArray`
        :class:`datetime.timedelta`    :class:`pandas.arrays.TimedeltaArray`
        :class:`int`                   :class:`pandas.arrays.IntegerArray`
        :class:`float`                 :class:`pandas.arrays.FloatingArray`
        :class:`str`                   :class:`pandas.arrays.StringArray` or
                                       :class:`pandas.arrays.ArrowStringArray`
        :class:`bool`                  :class:`pandas.arrays.BooleanArray`
        ============================== =======================================

        The ExtensionArray created when the scalar type is :class:`str` is determined by
        ``pd.options.mode.string_storage`` if the dtype is not explicitly given.

        For all other cases, NumPy's usual inference rules will be used.

        .. versionchanged:: 1.0.0

           Pandas infers nullable-integer dtype for integer data,
           string dtype for string data, and nullable-boolean dtype
           for boolean data.

        .. versionchanged:: 1.2.0

            Pandas now also infers nullable-floating dtype for float-like
            input data

    copy : bool, default True
        Whether to copy the data, even if not necessary. Depending
        on the type of `data`, creating the new array may require
        copying data, even if ``copy=False``.

    Returns
    -------
    ExtensionArray
        The newly created array.

    Raises
    ------
    ValueError
        When `data` is not 1-dimensional.

    See Also
    --------
    numpy.array : Construct a NumPy array.
    Series : Construct a pandas Series.
    Index : Construct a pandas Index.
    arrays.PandasArray : ExtensionArray wrapping a NumPy array.
    Series.array : Extract the array stored within a Series.

    Notes
    -----
    Omitting the `dtype` argument means pandas will attempt to infer the
    best array type from the values in the data. As new array types are
    added by pandas and 3rd party libraries, the "best" array type may
    change. We recommend specifying `dtype` to ensure that

    1. the correct array type for the data is returned
    2. the returned array type doesn't change as new extension types
       are added by pandas and third-party libraries

    Additionally, if the underlying memory representation of the returned
    array matters, we recommend specifying the `dtype` as a concrete object
    rather than a string alias or allowing it to be inferred. For example,
    a future version of pandas or a 3rd-party library may include a
    dedicated ExtensionArray for string data. In this event, the following
    would no longer return a :class:`arrays.PandasArray` backed by a NumPy
    array.

    >>> pd.array(['a', 'b'], dtype=str)
    <PandasArray>
    ['a', 'b']
    Length: 2, dtype: str32

    This would instead return the new ExtensionArray dedicated for string
    data. If you really need the new array to be backed by a  NumPy array,
    specify that in the dtype.

    >>> pd.array(['a', 'b'], dtype=np.dtype("<U1"))
    <PandasArray>
    ['a', 'b']
    Length: 2, dtype: str32

    Finally, Pandas has arrays that mostly overlap with NumPy

      * :class:`arrays.DatetimeArray`
      * :class:`arrays.TimedeltaArray`

    When data with a ``datetime64[ns]`` or ``timedelta64[ns]`` dtype is
    passed, pandas will always return a ``DatetimeArray`` or ``TimedeltaArray``
    rather than a ``PandasArray``. This is for symmetry with the case of
    timezone-aware data, which NumPy does not natively support.

    >>> pd.array(['2015', '2016'], dtype='datetime64[ns]')
    <DatetimeArray>
    ['2015-01-01 00:00:00', '2016-01-01 00:00:00']
    Length: 2, dtype: datetime64[ns]

    >>> pd.array(["1H", "2H"], dtype='timedelta64[ns]')
    <TimedeltaArray>
    ['0 days 01:00:00', '0 days 02:00:00']
    Length: 2, dtype: timedelta64[ns]

    Examples
    --------
    If a dtype is not specified, pandas will infer the best dtype from the values.
    See the description of `dtype` for the types pandas infers for.

    >>> pd.array([1, 2])
    <IntegerArray>
    [1, 2]
    Length: 2, dtype: Int64

    >>> pd.array([1, 2, np.nan])
    <IntegerArray>
    [1, 2, <NA>]
    Length: 3, dtype: Int64

    >>> pd.array([1.1, 2.2])
    <FloatingArray>
    [1.1, 2.2]
    Length: 2, dtype: Float64

    >>> pd.array(["a", None, "c"])
    <StringArray>
    ['a', <NA>, 'c']
    Length: 3, dtype: string

    >>> with pd.option_context("string_storage", "pyarrow"):
    ...     arr = pd.array(["a", None, "c"])
    ...
    >>> arr
    <ArrowStringArray>
    ['a', <NA>, 'c']
    Length: 3, dtype: string

    >>> pd.array([pd.Period('2000', freq="D"), pd.Period("2000", freq="D")])
    <PeriodArray>
    ['2000-01-01', '2000-01-01']
    Length: 2, dtype: period[D]

    You can use the string alias for `dtype`

    >>> pd.array(['a', 'b', 'a'], dtype='category')
    ['a', 'b', 'a']
    Categories (2, object): ['a', 'b']

    Or specify the actual dtype

    >>> pd.array(['a', 'b', 'a'],
    ...          dtype=pd.CategoricalDtype(['a', 'b', 'c'], ordered=True))
    ['a', 'b', 'a']
    Categories (3, object): ['a' < 'b' < 'c']

    If pandas does not infer a dedicated extension type a
    :class:`arrays.PandasArray` is returned.

    >>> pd.array([1 + 1j, 3 + 2j])
    <PandasArray>
    [(1+1j), (3+2j)]
    Length: 2, dtype: complex128

    As mentioned in the "Notes" section, new extension types may be added
    in the future (by pandas or 3rd party libraries), causing the return
    value to no longer be a :class:`arrays.PandasArray`. Specify the `dtype`
    as a NumPy dtype if you need to ensure there's no future change in
    behavior.

    >>> pd.array([1, 2], dtype=np.dtype("int32"))
    <PandasArray>
    [1, 2]
    Length: 2, dtype: int32

    `data` must be 1-dimensional. A ValueError is raised when the input
    has the wrong dimensionality.

    >>> pd.array(1)
    Traceback (most recent call last):
      ...
    ValueError: Cannot pass scalar '1' to 'pandas.array'.
    """
    ...

@overload
def extract_array(obj: Series | Index, extract_numpy: bool = ..., extract_range: bool = ...) -> ArrayLike:
    ...

@overload
def extract_array(obj: T, extract_numpy: bool = ..., extract_range: bool = ...) -> T | ArrayLike:
    ...

def extract_array(obj: T, extract_numpy: bool = ..., extract_range: bool = ...) -> T | ArrayLike:
    """
    Extract the ndarray or ExtensionArray from a Series or Index.

    For all other types, `obj` is just returned as is.

    Parameters
    ----------
    obj : object
        For Series / Index, the underlying ExtensionArray is unboxed.

    extract_numpy : bool, default False
        Whether to extract the ndarray from a PandasArray.

    extract_range : bool, default False
        If we have a RangeIndex, return range._values if True
        (which is a materialized integer ndarray), otherwise return unchanged.

    Returns
    -------
    arr : object

    Examples
    --------
    >>> extract_array(pd.Series(['a', 'b', 'c'], dtype='category'))
    ['a', 'b', 'c']
    Categories (3, object): ['a', 'b', 'c']

    Other objects like lists, arrays, and DataFrames are just passed through.

    >>> extract_array([1, 2, 3])
    [1, 2, 3]

    For an ndarray-backed Series / Index the ndarray is returned.

    >>> extract_array(pd.Series([1, 2, 3]))
    array([1, 2, 3])

    To extract all the way down to the ndarray, pass ``extract_numpy=True``.

    >>> extract_array(pd.Series([1, 2, 3]), extract_numpy=True)
    array([1, 2, 3])
    """
    ...

def ensure_wrapped_if_datetimelike(arr): # -> DatetimeArray | TimedeltaArray | ndarray[Unknown, Unknown]:
    """
    Wrap datetime64 and timedelta64 ndarrays in DatetimeArray/TimedeltaArray.
    """
    ...

def sanitize_masked_array(data: ma.MaskedArray) -> np.ndarray:
    """
    Convert numpy MaskedArray to ensure mask is softened.
    """
    ...

def sanitize_array(data, index: Index | None, dtype: DtypeObj | None = ..., copy: bool = ..., raise_cast_failure: bool = ..., *, allow_2d: bool = ...) -> ArrayLike:
    """
    Sanitize input data to an ndarray or ExtensionArray, copy if specified,
    coerce to the dtype if specified.

    Parameters
    ----------
    data : Any
    index : Index or None, default None
    dtype : np.dtype, ExtensionDtype, or None, default None
    copy : bool, default False
    raise_cast_failure : bool, default True
    allow_2d : bool, default False
        If False, raise if we have a 2D Arraylike.

    Returns
    -------
    np.ndarray or ExtensionArray

    Notes
    -----
    raise_cast_failure=False is only intended to be True when called from the
    DataFrame constructor, as the dtype keyword there may be interpreted as only
    applying to a subset of columns, see GH#24435.
    """
    ...

def range_to_ndarray(rng: range) -> np.ndarray:
    """
    Cast a range object to ndarray.
    """
    ...

def is_empty_data(data: Any) -> bool:
    """
    Utility to check if a Series is instantiated with empty data,
    which does not contain dtype information.

    Parameters
    ----------
    data : array-like, Iterable, dict, or scalar value
        Contains data stored in Series.

    Returns
    -------
    bool
    """
    ...

def create_series_with_explicit_dtype(data: Any = ..., index: ArrayLike | Index | None = ..., dtype: Dtype | None = ..., name: str | None = ..., copy: bool = ..., fastpath: bool = ..., dtype_if_empty: Dtype = ...) -> Series:
    """
    Helper to pass an explicit dtype when instantiating an empty Series.

    This silences a DeprecationWarning described in GitHub-17261.

    Parameters
    ----------
    data : Mirrored from Series.__init__
    index : Mirrored from Series.__init__
    dtype : Mirrored from Series.__init__
    name : Mirrored from Series.__init__
    copy : Mirrored from Series.__init__
    fastpath : Mirrored from Series.__init__
    dtype_if_empty : str, numpy.dtype, or ExtensionDtype
        This dtype will be passed explicitly if an empty Series will
        be instantiated.

    Returns
    -------
    Series
    """
    ...




"""
This type stub file was generated by pyright.
"""

from pandas.core.indexers import check_array_indexer
from pandas.core.indexers.objects import BaseIndexer, FixedForwardWindowIndexer, VariableOffsetWindowIndexer

"""
Public API for Rolling Window Indexers.
"""
__all__ = ["check_array_indexer", "BaseIndexer", "FixedForwardWindowIndexer", "VariableOffsetWindowIndexer"]



"""
This type stub file was generated by pyright.
"""

from pandas.compat.chainmap import DeepChainMap

"""
Module for scope operations
"""
def ensure_scope(level: int, global_dict=..., local_dict=..., resolvers=..., target=..., **kwargs) -> Scope:
    """Ensure that we are grabbing the correct scope."""
    ...

DEFAULT_GLOBALS = ...
class Scope:
    """
    Object to hold scope, with a few bells to deal with some custom syntax
    and contexts added by pandas.

    Parameters
    ----------
    level : int
    global_dict : dict or None, optional, default None
    local_dict : dict or Scope or None, optional, default None
    resolvers : list-like or None, optional, default None
    target : object

    Attributes
    ----------
    level : int
    scope : DeepChainMap
    target : object
    temps : dict
    """
    __slots__ = ...
    level: int
    scope: DeepChainMap
    resolvers: DeepChainMap
    temps: dict
    def __init__(self, level: int, global_dict=..., local_dict=..., resolvers=..., target=...) -> None:
        ...
    
    def __repr__(self) -> str:
        ...
    
    @property
    def has_resolvers(self) -> bool:
        """
        Return whether we have any extra scope.

        For example, DataFrames pass Their columns as resolvers during calls to
        ``DataFrame.eval()`` and ``DataFrame.query()``.

        Returns
        -------
        hr : bool
        """
        ...
    
    def resolve(self, key: str, is_local: bool):
        """
        Resolve a variable name in a possibly local context.

        Parameters
        ----------
        key : str
            A variable name
        is_local : bool
            Flag indicating whether the variable is local or not (prefixed with
            the '@' symbol)

        Returns
        -------
        value : object
            The value of a particular variable
        """
        ...
    
    def swapkey(self, old_key: str, new_key: str, new_value=...) -> None:
        """
        Replace a variable name, with a potentially new value.

        Parameters
        ----------
        old_key : str
            Current variable name to replace
        new_key : str
            New variable name to replace `old_key` with
        new_value : object
            Value to be replaced along with the possible renaming
        """
        ...
    
    def add_tmp(self, value) -> str:
        """
        Add a temporary variable to the scope.

        Parameters
        ----------
        value : object
            An arbitrary object to be assigned to a temporary variable.

        Returns
        -------
        str
            The name of the temporary variable created.
        """
        ...
    
    @property
    def ntemps(self) -> int:
        """The number of temporary variables in this scope"""
        ...
    
    @property
    def full_scope(self) -> DeepChainMap:
        """
        Return the full scope for use with passing to engines transparently
        as a mapping.

        Returns
        -------
        vars : DeepChainMap
            All variables in this scope.
        """
        ...
    





"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any, Callable, Literal
from pandas._typing import WindowingRankType, npt

def roll_sum(values: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int) -> np.ndarray:
    ...

def roll_mean(values: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int) -> np.ndarray:
    ...

def roll_var(values: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int, ddof: int = ...) -> np.ndarray:
    ...

def roll_skew(values: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int) -> np.ndarray:
    ...

def roll_kurt(values: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int) -> np.ndarray:
    ...

def roll_median_c(values: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int) -> np.ndarray:
    ...

def roll_max(values: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int) -> np.ndarray:
    ...

def roll_min(values: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int) -> np.ndarray:
    ...

def roll_quantile(values: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int, quantile: float, interpolation: Literal["linear", "lower", "higher", "nearest", "midpoint"]) -> np.ndarray:
    ...

def roll_rank(values: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int, percentile: bool, method: WindowingRankType, ascending: bool) -> np.ndarray:
    ...

def roll_apply(obj: object, start: np.ndarray, end: np.ndarray, minp: int, function: Callable[..., Any], raw: bool, args: tuple[Any, ...], kwargs: dict[str, Any]) -> npt.NDArray[np.float64]:
    ...

def roll_weighted_sum(values: np.ndarray, weights: np.ndarray, minp: int) -> np.ndarray:
    ...

def roll_weighted_mean(values: np.ndarray, weights: np.ndarray, minp: int) -> np.ndarray:
    ...

def roll_weighted_var(values: np.ndarray, weights: np.ndarray, minp: int, ddof: int) -> np.ndarray:
    ...

def ewm(vals: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int, com: float, adjust: bool, ignore_na: bool, deltas: np.ndarray, normalize: bool) -> np.ndarray:
    ...

def ewmcov(input_x: np.ndarray, start: np.ndarray, end: np.ndarray, minp: int, input_y: np.ndarray, com: float, adjust: bool, ignore_na: bool, bias: bool) -> np.ndarray:
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any
from pandas._typing import npt

class Infinity:
    """
    Provide a positive Infinity comparison method for ranking.
    """
    def __eq__(self, other) -> bool:
        ...
    
    def __ne__(self, other) -> bool:
        ...
    
    def __lt__(self, other) -> bool:
        ...
    
    def __le__(self, other) -> bool:
        ...
    
    def __gt__(self, other) -> bool:
        ...
    
    def __ge__(self, other) -> bool:
        ...
    


class NegInfinity:
    """
    Provide a negative Infinity comparison method for ranking.
    """
    def __eq__(self, other) -> bool:
        ...
    
    def __ne__(self, other) -> bool:
        ...
    
    def __lt__(self, other) -> bool:
        ...
    
    def __le__(self, other) -> bool:
        ...
    
    def __gt__(self, other) -> bool:
        ...
    
    def __ge__(self, other) -> bool:
        ...
    


def unique_deltas(arr: np.ndarray) -> np.ndarray:
    ...

def is_lexsorted(list_of_arrays: list[npt.NDArray[np.int64]]) -> bool:
    ...

def groupsort_indexer(index: np.ndarray, ngroups: int) -> tuple[np.ndarray, np.ndarray,]:
    ...

def kth_smallest(arr: np.ndarray, k: int) -> Any:
    ...

def nancorr(mat: npt.NDArray[np.float64], cov: bool = ..., minp: int | None = ...) -> npt.NDArray[np.float64]:
    ...

def nancorr_spearman(mat: npt.NDArray[np.float64], minp: int = ...) -> npt.NDArray[np.float64]:
    ...

def validate_limit(nobs: int | None, limit=...) -> int:
    ...

def pad(old: np.ndarray, new: np.ndarray, limit=...) -> npt.NDArray[np.intp]:
    ...

def pad_inplace(values: np.ndarray, mask: np.ndarray, limit=...) -> None:
    ...

def pad_2d_inplace(values: np.ndarray, mask: np.ndarray, limit=...) -> None:
    ...

def backfill(old: np.ndarray, new: np.ndarray, limit=...) -> npt.NDArray[np.intp]:
    ...

def backfill_inplace(values: np.ndarray, mask: np.ndarray, limit=...) -> None:
    ...

def backfill_2d_inplace(values: np.ndarray, mask: np.ndarray, limit=...) -> None:
    ...

def is_monotonic(arr: np.ndarray, timelike: bool) -> tuple[bool, bool, bool]:
    ...

def rank_1d(values: np.ndarray, labels: np.ndarray | None = ..., is_datetimelike: bool = ..., ties_method=..., ascending: bool = ..., pct: bool = ..., na_option=..., mask: npt.NDArray[np.bool_] | None = ...) -> np.ndarray:
    ...

def rank_2d(in_arr: np.ndarray, axis: int = ..., is_datetimelike: bool = ..., ties_method=..., ascending: bool = ..., na_option=..., pct: bool = ...) -> np.ndarray:
    ...

def diff_2d(arr: np.ndarray, out: np.ndarray, periods: int, axis: int, datetimelike: bool = ...) -> None:
    ...

def ensure_platform_int(arr: object) -> npt.NDArray[np.intp]:
    ...

def ensure_object(arr: object) -> npt.NDArray[np.object_]:
    ...

def ensure_float64(arr: object, copy=...) -> npt.NDArray[np.float64]:
    ...

def ensure_int8(arr: object, copy=...) -> npt.NDArray[np.int8]:
    ...

def ensure_int16(arr: object, copy=...) -> npt.NDArray[np.int16]:
    ...

def ensure_int32(arr: object, copy=...) -> npt.NDArray[np.int32]:
    ...

def ensure_int64(arr: object, copy=...) -> npt.NDArray[np.int64]:
    ...

def ensure_uint64(arr: object, copy=...) -> npt.NDArray[np.uint64]:
    ...

def take_1d_int8_int8(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int8_int32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int8_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int8_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int16_int16(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int16_int32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int16_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int16_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int32_int32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int32_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int32_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int64_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_int64_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_float32_float32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_float32_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_float64_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_object_object(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_bool_bool(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_1d_bool_object(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int8_int8(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int8_int32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int8_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int8_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int16_int16(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int16_int32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int16_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int16_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int32_int32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int32_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int32_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int64_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_int64_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_float32_float32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_float32_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_float64_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_object_object(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_bool_bool(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis0_bool_object(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int8_int8(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int8_int32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int8_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int8_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int16_int16(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int16_int32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int16_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int16_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int32_int32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int32_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int32_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int64_int64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_int64_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_float32_float32(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_float32_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_float64_float64(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_object_object(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_bool_bool(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_axis1_bool_object(values: np.ndarray, indexer: npt.NDArray[np.intp], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int8_int8(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int8_int32(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int8_int64(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int8_float64(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int16_int16(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int16_int32(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int16_int64(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int16_float64(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int32_int32(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int32_int64(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int32_float64(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int64_float64(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_float32_float32(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_float32_float64(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_float64_float64(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_object_object(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_bool_bool(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_bool_object(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...

def take_2d_multi_int64_int64(values: np.ndarray, indexer: tuple[npt.NDArray[np.intp], npt.NDArray[np.intp]], out: np.ndarray, fill_value=...) -> None:
    ...




"""
This type stub file was generated by pyright.
"""

from pandas.core.internals.api import make_block
from pandas.core.internals.array_manager import ArrayManager, SingleArrayManager
from pandas.core.internals.base import DataManager, SingleDataManager
from pandas.core.internals.blocks import Block, DatetimeTZBlock, ExtensionBlock, NumericBlock, ObjectBlock
from pandas.core.internals.concat import concatenate_managers
from pandas.core.internals.managers import BlockManager, SingleBlockManager, create_block_manager_from_blocks

__all__ = ["Block", "NumericBlock", "DatetimeTZBlock", "ExtensionBlock", "ObjectBlock", "make_block", "DataManager", "ArrayManager", "BlockManager", "SingleDataManager", "SingleBlockManager", "SingleArrayManager", "concatenate_managers", "create_block_manager_from_blocks"]
def __getattr__(name: str): # -> Type[CategoricalBlock]:
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Any, Callable, Iterable, Literal, overload
from pandas._typing import npt

_BinOp = Callable[[Any, Any], Any]
_BoolOp = Callable[[Any, Any], bool]
def scalar_compare(values: np.ndarray, val: object, op: _BoolOp) -> npt.NDArray[np.bool_]:
    ...

def vec_compare(left: npt.NDArray[np.object_], right: npt.NDArray[np.object_], op: _BoolOp) -> npt.NDArray[np.bool_]:
    ...

def scalar_binop(values: np.ndarray, val: object, op: _BinOp) -> np.ndarray:
    ...

def vec_binop(left: np.ndarray, right: np.ndarray, op: _BinOp) -> np.ndarray:
    ...

@overload
def maybe_convert_bool(arr: npt.NDArray[np.object_], true_values: Iterable = ..., false_values: Iterable = ..., convert_to_masked_nullable: Literal[False] = ...) -> tuple[np.ndarray, None]:
    ...

@overload
def maybe_convert_bool(arr: npt.NDArray[np.object_], true_values: Iterable = ..., false_values: Iterable = ..., *, convert_to_masked_nullable: Literal[True]) -> tuple[np.ndarray, np.ndarray]:
    ...




"""
This type stub file was generated by pyright.
"""

import weakref
import numpy as np
from typing import Any, Callable, Hashable, Literal, Sequence, TypeVar
from pandas._libs import internals as libinternals
from pandas._typing import ArrayLike, DtypeObj, Shape, npt, type_t
from pandas.core.indexes.api import Float64Index, Index
from pandas.core.internals.base import DataManager, SingleDataManager
from pandas.core.internals.blocks import Block

T = TypeVar("T", bound="BaseBlockManager")
class BaseBlockManager(DataManager):
    """
    Core internal data structure to implement DataFrame, Series, etc.

    Manage a bunch of labeled 2D mixed-type ndarrays. Essentially it's a
    lightweight blocked set of labeled data to be manipulated by the DataFrame
    public API class

    Attributes
    ----------
    shape
    ndim
    axes
    values
    items

    Methods
    -------
    set_axis(axis, new_labels)
    copy(deep=True)

    get_dtypes

    apply(func, axes, block_filter_fn)

    get_bool_data
    get_numeric_data

    get_slice(slice_like, axis)
    get(label)
    iget(loc)

    take(indexer, axis)
    reindex_axis(new_labels, axis)
    reindex_indexer(new_labels, indexer, axis)

    delete(label)
    insert(loc, label, value)
    set(label, value)

    Parameters
    ----------
    blocks: Sequence of Block
    axes: Sequence of Index
    verify_integrity: bool, default True

    Notes
    -----
    This is *not* a public API class
    """
    __slots__ = ...
    _blknos: npt.NDArray[np.intp]
    _blklocs: npt.NDArray[np.intp]
    blocks: tuple[Block, ...]
    axes: list[Index]
    refs: list[weakref.ref | None] | None
    @property
    def ndim(self) -> int:
        ...
    
    _known_consolidated: bool
    _is_consolidated: bool
    def __init__(self, blocks, axes, refs=..., verify_integrity: bool = ...) -> None:
        ...
    
    @classmethod
    def from_blocks(cls: type_t[T], blocks: list[Block], axes: list[Index], refs: list[weakref.ref | None] | None = ...) -> T:
        ...
    
    @property
    def blknos(self) -> npt.NDArray[np.intp]:
        """
        Suppose we want to find the array corresponding to our i'th column.

        blknos[i] identifies the block from self.blocks that contains this column.

        blklocs[i] identifies the column of interest within
        self.blocks[self.blknos[i]]
        """
        ...
    
    @property
    def blklocs(self) -> npt.NDArray[np.intp]:
        """
        See blknos.__doc__
        """
        ...
    
    def make_empty(self: T, axes=...) -> T:
        """return an empty BlockManager with the items axis of len 0"""
        ...
    
    def __nonzero__(self) -> bool:
        ...
    
    __bool__ = ...
    def set_axis(self, axis: int, new_labels: Index) -> None:
        ...
    
    @property
    def is_single_block(self) -> bool:
        ...
    
    @property
    def items(self) -> Index:
        ...
    
    def get_dtypes(self): # -> ndarray[Any, dtype[Any]]:
        ...
    
    @property
    def arrays(self) -> list[ArrayLike]:
        """
        Quick access to the backing arrays of the Blocks.

        Only for compatibility with ArrayManager for testing convenience.
        Not to be used in actual code, and return value is not the same as the
        ArrayManager method (list of 1D arrays vs iterator of 2D ndarrays / 1D EAs).

        Warning! The returned arrays don't handle Copy-on-Write, so this should
        be used with caution (only in read-mode).
        """
        ...
    
    def __repr__(self) -> str:
        ...
    
    def apply(self: T, f, align_keys: list[str] | None = ..., ignore_failures: bool = ..., **kwargs) -> T:
        """
        Iterate over the blocks, collect and create a new BlockManager.

        Parameters
        ----------
        f : str or callable
            Name of the Block method to apply.
        align_keys: List[str] or None, default None
        ignore_failures: bool, default False
        **kwargs
            Keywords to pass to `f`

        Returns
        -------
        BlockManager
        """
        ...
    
    def where(self: T, other, cond, align: bool) -> T:
        ...
    
    def setitem(self: T, indexer, value) -> T:
        """
        Set values with indexer.

        For SingleBlockManager, this backs s[indexer] = value
        """
        ...
    
    def putmask(self, mask, new, align: bool = ...): # -> Self@BaseBlockManager:
        ...
    
    def diff(self: T, n: int, axis: int) -> T:
        ...
    
    def interpolate(self: T, **kwargs) -> T:
        ...
    
    def shift(self: T, periods: int, axis: int, fill_value) -> T:
        ...
    
    def fillna(self: T, value, limit, inplace: bool, downcast) -> T:
        ...
    
    def astype(self: T, dtype, copy: bool = ..., errors: str = ...) -> T:
        ...
    
    def convert(self: T, copy: bool = ..., datetime: bool = ..., numeric: bool = ..., timedelta: bool = ...) -> T:
        ...
    
    def replace(self: T, to_replace, value, inplace: bool) -> T:
        ...
    
    def replace_regex(self, **kwargs): # -> Self@BaseBlockManager:
        ...
    
    def replace_list(self: T, src_list: list[Any], dest_list: list[Any], inplace: bool = ..., regex: bool = ...) -> T:
        """do a list replace"""
        ...
    
    def to_native_types(self: T, **kwargs) -> T:
        """
        Convert values to native types (strings / python objects) that are used
        in formatting (repr / csv).
        """
        ...
    
    @property
    def is_numeric_mixed_type(self) -> bool:
        ...
    
    @property
    def any_extension_types(self) -> bool:
        """Whether any of the blocks in this manager are extension blocks"""
        ...
    
    @property
    def is_view(self) -> bool:
        """return a boolean if we are a single block and are a view"""
        ...
    
    def get_bool_data(self: T, copy: bool = ...) -> T:
        """
        Select blocks that are bool-dtype and columns from object-dtype blocks
        that are all-bool.

        Parameters
        ----------
        copy : bool, default False
            Whether to copy the blocks
        """
        ...
    
    def get_numeric_data(self: T, copy: bool = ...) -> T:
        """
        Parameters
        ----------
        copy : bool, default False
            Whether to copy the blocks
        """
        ...
    
    @property
    def nblocks(self) -> int:
        ...
    
    def copy(self: T, deep=...) -> T:
        """
        Make deep or shallow copy of BlockManager

        Parameters
        ----------
        deep : bool, string or None, default True
            If False or None, return a shallow copy (do not copy data)
            If 'all', copy data and a deep copy of the index

        Returns
        -------
        BlockManager
        """
        ...
    
    def consolidate(self: T) -> T:
        """
        Join together blocks having same dtype

        Returns
        -------
        y : BlockManager
        """
        ...
    
    def reindex_indexer(self: T, new_axis: Index, indexer: npt.NDArray[np.intp] | None, axis: int, fill_value=..., allow_dups: bool = ..., copy: bool | None = ..., only_slice: bool = ..., *, use_na_proxy: bool = ...) -> T:
        """
        Parameters
        ----------
        new_axis : Index
        indexer : ndarray[intp] or None
        axis : int
        fill_value : object, default None
        allow_dups : bool, default False
        copy : bool or None, default True
            If None, regard as False to get shallow copy.
        only_slice : bool, default False
            Whether to take views, not copies, along columns.
        use_na_proxy : bool, default False
            Whether to use a np.void ndarray for newly introduced columns.

        pandas-indexer with -1's only.
        """
        ...
    
    def take(self: T, indexer, axis: int = ..., verify: bool = ..., convert_indices: bool = ...) -> T:
        """
        Take items along any axis.

        indexer : np.ndarray or slice
        axis : int, default 1
        verify : bool, default True
            Check that all entries are between 0 and len(self) - 1, inclusive.
            Pass verify=False if this check has been done by the caller.
        convert_indices : bool, default True
            Whether to attempt to convert indices to positive values.

        Returns
        -------
        BlockManager
        """
        ...
    


class BlockManager(libinternals.BlockManager, BaseBlockManager):
    """
    BaseBlockManager that holds 2D blocks.
    """
    ndim = ...
    def __init__(self, blocks: Sequence[Block], axes: Sequence[Index], refs: list[weakref.ref | None] | None = ..., verify_integrity: bool = ...) -> None:
        ...
    
    @classmethod
    def from_blocks(cls, blocks: list[Block], axes: list[Index], refs: list[weakref.ref | None] | None = ...) -> BlockManager:
        """
        Constructor for BlockManager and SingleBlockManager with same signature.
        """
        ...
    
    def fast_xs(self, loc: int) -> SingleBlockManager:
        """
        Return the array corresponding to `frame.iloc[loc]`.

        Parameters
        ----------
        loc : int

        Returns
        -------
        np.ndarray or ExtensionArray
        """
        ...
    
    def iget(self, i: int) -> SingleBlockManager:
        """
        Return the data as a SingleBlockManager.
        """
        ...
    
    def iget_values(self, i: int) -> ArrayLike:
        """
        Return the data for column i as the values (ndarray or ExtensionArray).

        Warning! The returned array is a view but doesn't handle Copy-on-Write,
        so this should be used with caution.
        """
        ...
    
    @property
    def column_arrays(self) -> list[np.ndarray]:
        """
        Used in the JSON C code to access column arrays.
        This optimizes compared to using `iget_values` by converting each

        Warning! This doesn't handle Copy-on-Write, so should be used with
        caution (current use case of consuming this in the JSON code is fine).
        """
        ...
    
    def iset(self, loc: int | slice | np.ndarray, value: ArrayLike, inplace: bool = ...): # -> None:
        """
        Set new item in-place. Does not consolidate. Adds new Block if not
        contained in the current set of items
        """
        ...
    
    def column_setitem(self, loc: int, idx: int | slice | np.ndarray, value) -> None:
        """
        Set values ("setitem") into a single column (not setting the full column).

        This is a method on the BlockManager level, to avoid creating an
        intermediate Series at the DataFrame level (`s = df[loc]; s[idx] = value`)
        """
        ...
    
    def insert(self, loc: int, item: Hashable, value: ArrayLike) -> None:
        """
        Insert item at selected position.

        Parameters
        ----------
        loc : int
        item : hashable
        value : np.ndarray or ExtensionArray
        """
        ...
    
    def idelete(self, indexer) -> BlockManager:
        """
        Delete selected locations, returning a new BlockManager.
        """
        ...
    
    def grouped_reduce(self: T, func: Callable, ignore_failures: bool = ...) -> T:
        """
        Apply grouped reduction function blockwise, returning a new BlockManager.

        Parameters
        ----------
        func : grouped reduction function
        ignore_failures : bool, default False
            Whether to drop blocks where func raises TypeError.

        Returns
        -------
        BlockManager
        """
        ...
    
    def reduce(self: T, func: Callable, ignore_failures: bool = ...) -> tuple[T, np.ndarray]:
        """
        Apply reduction function blockwise, returning a single-row BlockManager.

        Parameters
        ----------
        func : reduction function
        ignore_failures : bool, default False
            Whether to drop blocks where func raises TypeError.

        Returns
        -------
        BlockManager
        np.ndarray
            Indexer of mgr_locs that are retained.
        """
        ...
    
    def operate_blockwise(self, other: BlockManager, array_op) -> BlockManager:
        """
        Apply array_op blockwise with another (aligned) BlockManager.
        """
        ...
    
    def quantile(self: T, *, qs: Float64Index, axis: int = ..., interpolation=...) -> T:
        """
        Iterate over blocks applying quantile reduction.
        This routine is intended for reduction type operations and
        will do inference on the generated blocks.

        Parameters
        ----------
        axis: reduction axis, default 0
        consolidate: bool, default True. Join together blocks having same
            dtype
        interpolation : type of interpolation, default 'linear'
        qs : list of the quantiles to be computed

        Returns
        -------
        BlockManager
        """
        ...
    
    def unstack(self, unstacker, fill_value) -> BlockManager:
        """
        Return a BlockManager with all blocks unstacked.

        Parameters
        ----------
        unstacker : reshape._Unstacker
        fill_value : Any
            fill_value for newly introduced missing values.

        Returns
        -------
        unstacked : BlockManager
        """
        ...
    
    def to_dict(self, copy: bool = ...): # -> dict[str, Self@BlockManager]:
        """
        Return a dict of str(dtype) -> BlockManager

        Parameters
        ----------
        copy : bool, default True

        Returns
        -------
        values : a dict of dtype -> BlockManager
        """
        ...
    
    def as_array(self, dtype: np.dtype | None = ..., copy: bool = ..., na_value: object = ...) -> np.ndarray:
        """
        Convert the blockmanager data into an numpy array.

        Parameters
        ----------
        dtype : np.dtype or None, default None
            Data type of the return array.
        copy : bool, default False
            If True then guarantee that a copy is returned. A value of
            False does not guarantee that the underlying data is not
            copied.
        na_value : object, default lib.no_default
            Value to be used as the missing value sentinel.

        Returns
        -------
        arr : ndarray
        """
        ...
    
    def is_consolidated(self) -> bool:
        """
        Return True if more than one block with the same dtype
        """
        ...
    


class SingleBlockManager(BaseBlockManager, SingleDataManager):
    """manage a single block with"""
    @property
    def ndim(self) -> Literal[1]:
        ...
    
    _is_consolidated = ...
    _known_consolidated = ...
    __slots__ = ...
    is_single_block = ...
    def __init__(self, block: Block, axis: Index, refs: list[weakref.ref | None] | None = ..., verify_integrity: bool = ..., fastpath=...) -> None:
        ...
    
    @classmethod
    def from_blocks(cls, blocks: list[Block], axes: list[Index], refs: list[weakref.ref | None] | None = ...) -> SingleBlockManager:
        """
        Constructor for BlockManager and SingleBlockManager with same signature.
        """
        ...
    
    @classmethod
    def from_array(cls, array: ArrayLike, index: Index) -> SingleBlockManager:
        """
        Constructor for if we have an array that is not yet a Block.
        """
        ...
    
    def to_2d_mgr(self, columns: Index) -> BlockManager:
        """
        Manager analogue of Series.to_frame
        """
        ...
    
    def __getstate__(self): # -> tuple[list[Index], list[ndarray[Unknown, Unknown] | ExtensionArray], list[Any | Index | Unknown | ExtensionArray], dict[str, dict[str, Unknown]]]:
        ...
    
    def __setstate__(self, state): # -> None:
        ...
    
    def getitem_mgr(self, indexer: slice | npt.NDArray[np.bool_]) -> SingleBlockManager:
        ...
    
    def get_slice(self, slobj: slice, axis: int = ...) -> SingleBlockManager:
        ...
    
    @property
    def index(self) -> Index:
        ...
    
    @property
    def dtype(self) -> DtypeObj:
        ...
    
    def get_dtypes(self) -> np.ndarray:
        ...
    
    def external_values(self): # -> ArrayLike:
        """The array that Series.values returns"""
        ...
    
    def internal_values(self): # -> ndarray[Unknown, Unknown] | ExtensionArray:
        """The array that Series._values returns"""
        ...
    
    def array_values(self): # -> ExtensionArray:
        """The array that Series.array returns"""
        ...
    
    def get_numeric_data(self, copy: bool = ...): # -> Self@SingleBlockManager:
        ...
    
    def setitem_inplace(self, indexer, value) -> None:
        """
        Set values with indexer.

        For Single[Block/Array]Manager, this backs s[indexer] = value

        This is an inplace version of `setitem()`, mutating the manager/values
        in place, not returning a new Manager (and Block), and thus never changing
        the dtype.
        """
        ...
    
    def idelete(self, indexer) -> SingleBlockManager:
        """
        Delete single location from SingleBlockManager.

        Ensures that self.blocks doesn't become empty.
        """
        ...
    
    def fast_xs(self, loc):
        """
        fast path for getting a cross-section
        return a view of the data
        """
        ...
    
    def set_values(self, values: ArrayLike): # -> None:
        """
        Set the values of the single block in place.

        Use at your own risk! This does not check if the passed values are
        valid for the current Block/SingleBlockManager (length, dtype, etc).
        """
        ...
    


def create_block_manager_from_blocks(blocks: list[Block], axes: list[Index], consolidate: bool = ..., verify_integrity: bool = ...) -> BlockManager:
    ...

def create_block_manager_from_column_arrays(arrays: list[ArrayLike], axes: list[Index], consolidate: bool = ...) -> BlockManager:
    ...

def construction_error(tot_items: int, block_shape: Shape, axes: list[Index], e: ValueError | None = ...): # -> ValueError:
    """raise a helpful message about our construction"""
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas.core.interchange.dataframe_protocol import Buffer, DlpackDeviceType

_NUMPY_HAS_DLPACK = ...
class PandasBuffer(Buffer):
    """
    Data in the buffer is guaranteed to be contiguous in memory.
    """
    def __init__(self, x: np.ndarray, allow_copy: bool = ...) -> None:
        """
        Handle only regular columns (= numpy arrays) for now.
        """
        ...
    
    @property
    def bufsize(self) -> int:
        """
        Buffer size in bytes.
        """
        ...
    
    @property
    def ptr(self) -> int:
        """
        Pointer to start of the buffer as an integer.
        """
        ...
    
    def __dlpack__(self): # -> _PyCapsule:
        """
        Represent this structure as DLPack interface.
        """
        ...
    
    def __dlpack_device__(self) -> tuple[DlpackDeviceType, int | None]:
        """
        Device type and device ID for where the data in the buffer resides.
        """
        ...
    
    def __repr__(self) -> str:
        ...
    





"""
This type stub file was generated by pyright.
"""

import inspect
import contextlib
import ctypes
import os
import platform
import subprocess
import time
import warnings
from ctypes import c_size_t, c_wchar, c_wchar_p, get_errno, sizeof
from shutil import which
from pandas.errors import PyperclipException, PyperclipWindowsException
from pandas.util._exceptions import find_stack_level

"""
Pyperclip

A cross-platform clipboard module for Python,
with copy & paste functions for plain text.
By Al Sweigart al@inventwithpython.com
BSD License

Usage:
  import pyperclip
  pyperclip.copy('The text to be copied to the clipboard.')
  spam = pyperclip.paste()

  if not pyperclip.is_available():
    print("Copy functionality unavailable!")

On Windows, no additional modules are needed.
On Mac, the pyobjc module is used, falling back to the pbcopy and pbpaste cli
    commands. (These commands should come with OS X.).
On Linux, install xclip or xsel via package manager. For example, in Debian:
    sudo apt-get install xclip
    sudo apt-get install xsel

Otherwise on Linux, you will need the PyQt5 modules installed.

This module does not work with PyGObject yet.

Cygwin is currently not supported.

Security Note: This module runs programs with these names:
    - which
    - where
    - pbcopy
    - pbpaste
    - xclip
    - xsel
    - klipper
    - qdbus
A malicious user could rename or add programs with these names, tricking
Pyperclip into running them with whatever permissions the Python process has.

"""
__version__ = ...
HAS_DISPLAY = ...
EXCEPT_MSG = ...
ENCODING = ...
if platform.system() == "Windows":
    WHICH_CMD = ...
else:
    WHICH_CMD = ...
def init_osx_pbcopy_clipboard(): # -> tuple[(text: Unknown) -> None, () -> str]:
    ...

def init_osx_pyobjc_clipboard(): # -> tuple[(text: Unknown) -> None, () -> Unknown]:
    ...

def init_qt_clipboard(): # -> tuple[(text: Unknown) -> None, () -> str]:
    ...

def init_xclip_clipboard(): # -> tuple[(text: Unknown, primary: bool = False) -> None, (primary: bool = False) -> str]:
    ...

def init_xsel_clipboard(): # -> tuple[(text: Unknown, primary: bool = False) -> None, (primary: bool = False) -> str]:
    ...

def init_klipper_clipboard(): # -> tuple[(text: Unknown) -> None, () -> str]:
    ...

def init_dev_clipboard_clipboard(): # -> tuple[(text: Unknown) -> None, () -> str]:
    ...

def init_no_clipboard(): # -> tuple[ClipboardUnavailable, ClipboardUnavailable]:
    class ClipboardUnavailable:
        ...
    
    

class CheckedCall:
    def __init__(self, f) -> None:
        ...
    
    def __call__(self, *args):
        ...
    
    def __setattr__(self, key, value): # -> None:
        ...
    


def init_windows_clipboard(): # -> tuple[(text: Unknown) -> None, () -> (str | None)]:
    ...

def init_wsl_clipboard(): # -> tuple[(text: Unknown) -> None, () -> str]:
    ...

def determine_clipboard(): # -> tuple[(text: Unknown) -> None, () -> str] | tuple[(text: Unknown) -> None, () -> (str | None)] | tuple[(text: Unknown, primary: bool = False) -> None, (primary: bool = False) -> str] | tuple[ClipboardUnavailable, ClipboardUnavailable]:
    """
    Determine the OS/platform and set the copy() and paste() functions
    accordingly.
    """
    ...

def set_clipboard(clipboard): # -> None:
    """
    Explicitly sets the clipboard mechanism. The "clipboard mechanism" is how
    the copy() and paste() functions interact with the operating system to
    implement the copy/paste feature. The clipboard parameter must be one of:
        - pbcopy
        - pbobjc (default on Mac OS X)
        - qt
        - xclip
        - xsel
        - klipper
        - windows (default on Windows)
        - no (this is what is set when no clipboard mechanism can be found)
    """
    ...

def lazy_load_stub_copy(text): # -> None:
    """
    A stub function for copy(), which will load the real copy() function when
    called so that the real copy() function is used for later calls.

    This allows users to import pyperclip without having determine_clipboard()
    automatically run, which will automatically select a clipboard mechanism.
    This could be a problem if it selects, say, the memory-heavy PyQt4 module
    but the user was just going to immediately call set_clipboard() to use a
    different clipboard mechanism.

    The lazy loading this stub function implements gives the user a chance to
    call set_clipboard() to pick another clipboard mechanism. Or, if the user
    simply calls copy() or paste() without calling set_clipboard() first,
    will fall back on whatever clipboard mechanism that determine_clipboard()
    automatically chooses.
    """
    ...

def lazy_load_stub_paste(): # -> str | None:
    """
    A stub function for paste(), which will load the real paste() function when
    called so that the real paste() function is used for later calls.

    This allows users to import pyperclip without having determine_clipboard()
    automatically run, which will automatically select a clipboard mechanism.
    This could be a problem if it selects, say, the memory-heavy PyQt4 module
    but the user was just going to immediately call set_clipboard() to use a
    different clipboard mechanism.

    The lazy loading this stub function implements gives the user a chance to
    call set_clipboard() to pick another clipboard mechanism. Or, if the user
    simply calls copy() or paste() without calling set_clipboard() first,
    will fall back on whatever clipboard mechanism that determine_clipboard()
    automatically chooses.
    """
    ...

def is_available() -> bool:
    ...

__all__ = ["copy", "paste", "set_clipboard", "determine_clipboard"]
clipboard_get = ...
clipboard_set = ...



"""
This type stub file was generated by pyright.
"""

import inspect
import operator
import warnings
import numpy as np
from __future__ import annotations
from typing import TYPE_CHECKING
from pandas._libs.ops_dispatch import maybe_dispatch_ufunc_to_dunder_op
from pandas._typing import Level
from pandas.util._decorators import Appender
from pandas.util._exceptions import find_stack_level
from pandas.core.dtypes.common import is_array_like, is_list_like
from pandas.core.dtypes.generic import ABCDataFrame, ABCSeries
from pandas.core.dtypes.missing import isna
from pandas.core import algorithms, roperator
from pandas.core.ops.array_ops import arithmetic_op, comp_method_OBJECT_ARRAY, comparison_op, get_array_op, logical_op, maybe_prepare_scalar_for_op
from pandas.core.ops.common import get_op_result_name, unpack_zerodim_and_defer
from pandas.core.ops.docstrings import _flex_comp_doc_FRAME, _op_descriptions, make_flex_doc
from pandas.core.ops.invalid import invalid_comparison
from pandas.core.ops.mask_ops import kleene_and, kleene_or, kleene_xor
from pandas.core.ops.methods import add_flex_arithmetic_methods
from pandas.core.roperator import radd, rand_, rdiv, rdivmod, rfloordiv, rmod, rmul, ror_, rpow, rsub, rtruediv, rxor
from pandas import DataFrame, Series

"""
Arithmetic operations for PandasObjects

This is not a public API.
"""
if TYPE_CHECKING:
    ...
ARITHMETIC_BINOPS: set[str] = ...
COMPARISON_BINOPS: set[str] = ...
def fill_binop(left, right, fill_value): # -> tuple[Unknown, Unknown]:
    """
    If a non-None fill_value is given, replace null entries in left and right
    with this value, but only in positions where _one_ of left/right is null,
    not both.

    Parameters
    ----------
    left : array-like
    right : array-like
    fill_value : object

    Returns
    -------
    left : array-like
    right : array-like

    Notes
    -----
    Makes copies if fill_value is not None and NAs are present.
    """
    ...

def align_method_SERIES(left: Series, right, align_asobject: bool = ...): # -> tuple[Series, Any | Unknown | ABCSeries]:
    """align lhs and rhs Series"""
    ...

def flex_method_SERIES(op): # -> (self: Unknown, other: Unknown, level: Unknown | None = None, fill_value: Unknown | None = None, axis: int = 0) -> Unknown:
    ...

def align_method_FRAME(left, right, axis, flex: bool | None = ..., level: Level = ...): # -> tuple[Unknown, Unknown | ABCDataFrame | Series | ndarray[Unknown, Unknown]]:
    """
    Convert rhs to meet lhs dims if input is list, tuple or np.ndarray.

    Parameters
    ----------
    left : DataFrame
    right : Any
    axis : int, str, or None
    flex : bool or None, default False
        Whether this is a flex op, in which case we reindex.
        None indicates not to check for alignment.
    level : int or level name, default None

    Returns
    -------
    left : DataFrame
    right : Any
    """
    ...

def should_reindex_frame_op(left: DataFrame, right, op, axis, default_axis, fill_value, level) -> bool:
    """
    Check if this is an operation between DataFrames that will need to reindex.
    """
    ...

def frame_arith_method_with_reindex(left: DataFrame, right: DataFrame, op) -> DataFrame:
    """
    For DataFrame-with-DataFrame operations that require reindexing,
    operate only on shared columns, then reindex.

    Parameters
    ----------
    left : DataFrame
    right : DataFrame
    op : binary operator

    Returns
    -------
    DataFrame
    """
    ...

def flex_arith_method_FRAME(op): # -> (self: Unknown, other: Unknown, axis: str = default_axis, level: Unknown | None = None, fill_value: Unknown | None = None) -> (DataFrame | Unknown):
    ...

def flex_comp_method_FRAME(op): # -> (self: Unknown, other: Unknown, axis: str = default_axis, level: Unknown | None = None) -> Unknown:
    ...

__all__ = ["add_flex_arithmetic_methods", "align_method_FRAME", "align_method_SERIES", "ARITHMETIC_BINOPS", "arithmetic_op", "COMPARISON_BINOPS", "comparison_op", "comp_method_OBJECT_ARRAY", "fill_binop", "flex_arith_method_FRAME", "flex_comp_method_FRAME", "flex_method_SERIES", "frame_arith_method_with_reindex", "invalid_comparison", "kleene_and", "kleene_or", "kleene_xor", "logical_op", "maybe_dispatch_ufunc_to_dunder_op", "radd", "rand_", "rdiv", "rdivmod", "rfloordiv", "rmod", "rmul", "ror_", "rpow", "rsub", "rtruediv", "rxor", "should_reindex_frame_op", "unpack_zerodim_and_defer"]



"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING
from pandas.core.strings.base import BaseStringArrayMethods

if TYPE_CHECKING:
    ...
class ObjectStringArrayMixin(BaseStringArrayMethods):
    """
    String Methods operating on object-dtype ndarrays.
    """
    _str_na_value = ...
    def __len__(self):
        ...
    





"""
This type stub file was generated by pyright.
"""

from pandas._libs.tslibs import dtypes
from pandas._libs.tslibs.conversion import localize_pydatetime
from pandas._libs.tslibs.dtypes import Resolution, is_supported_unit, periods_per_day, periods_per_second
from pandas._libs.tslibs.nattype import NaT, NaTType, iNaT, nat_strings
from pandas._libs.tslibs.np_datetime import OutOfBoundsDatetime, OutOfBoundsTimedelta, astype_overflowsafe, is_unitless, py_get_unit_from_dtype as get_unit_from_dtype
from pandas._libs.tslibs.offsets import BaseOffset, Tick, to_offset
from pandas._libs.tslibs.period import IncompatibleFrequency, Period
from pandas._libs.tslibs.timedeltas import Timedelta, delta_to_nanoseconds, ints_to_pytimedelta
from pandas._libs.tslibs.timestamps import Timestamp
from pandas._libs.tslibs.timezones import tz_compare
from pandas._libs.tslibs.tzconversion import tz_convert_from_utc_single
from pandas._libs.tslibs.vectorized import dt64arr_to_periodarr, get_resolution, ints_to_pydatetime, is_date_array_normalized, normalize_i8_timestamps, tz_convert_from_utc

__all__ = ["dtypes", "localize_pydatetime", "NaT", "NaTType", "iNaT", "nat_strings", "OutOfBoundsDatetime", "OutOfBoundsTimedelta", "IncompatibleFrequency", "Period", "Resolution", "Timedelta", "normalize_i8_timestamps", "is_date_array_normalized", "dt64arr_to_periodarr", "delta_to_nanoseconds", "ints_to_pydatetime", "ints_to_pytimedelta", "get_resolution", "Timestamp", "tz_convert_from_utc_single", "tz_convert_from_utc", "to_offset", "Tick", "BaseOffset", "tz_compare", "is_unitless", "astype_overflowsafe", "get_unit_from_dtype", "periods_per_day", "periods_per_second", "is_supported_unit"]



"""
This type stub file was generated by pyright.
"""

from collections import abc
from typing import Hashable, IO, Iterator, Literal, Mapping, Sequence, TYPE_CHECKING
from pandas._typing import ArrayLike, ReadCsvBuffer
from pandas.io.parsers.base_parser import ParserBase
from pandas import Index, MultiIndex

if TYPE_CHECKING:
    ...
_BOM = ...
class PythonParser(ParserBase):
    def __init__(self, f: ReadCsvBuffer[str] | list, **kwds) -> None:
        """
        Workhorse function for processing nested list into DataFrame
        """
        ...
    
    def read(self, rows: int | None = ...) -> tuple[Index | None, Sequence[Hashable] | MultiIndex, Mapping[Hashable, ArrayLike]]:
        ...
    
    def get_chunk(self, size: int | None = ...) -> tuple[Index | None, Sequence[Hashable] | MultiIndex, Mapping[Hashable, ArrayLike]]:
        ...
    
    _implicit_index = ...


class FixedWidthReader(abc.Iterator):
    """
    A reader of fixed-width lines.
    """
    def __init__(self, f: IO[str] | ReadCsvBuffer[str], colspecs: list[tuple[int, int]] | Literal["infer"], delimiter: str | None, comment: str | None, skiprows: set[int] | None = ..., infer_nrows: int = ...) -> None:
        ...
    
    def get_rows(self, infer_nrows: int, skiprows: set[int] | None = ...) -> list[str]:
        """
        Read rows from self.f, skipping as specified.

        We distinguish buffer_rows (the first <= infer_nrows
        lines) from the rows returned to detect_colspecs
        because it's simpler to leave the other locations
        with skiprows logic alone than to modify them to
        deal with the fact we skipped some rows here as
        well.

        Parameters
        ----------
        infer_nrows : int
            Number of rows to read from self.f, not counting
            rows that are skipped.
        skiprows: set, optional
            Indices of rows to skip.

        Returns
        -------
        detect_rows : list of str
            A list containing the rows to read.

        """
        ...
    
    def detect_colspecs(self, infer_nrows: int = ..., skiprows: set[int] | None = ...) -> list[tuple[int, int]]:
        ...
    
    def __next__(self) -> list[str]:
        ...
    


class FixedWidthFieldParser(PythonParser):
    """
    Specialization that Converts fixed-width fields into DataFrames.
    See PythonParser for details.
    """
    def __init__(self, f: ReadCsvBuffer[str], **kwds) -> None:
        ...
    


def count_empty_vals(vals) -> int:
    ...




# # BoARIO : The Adaptative Regional Input Output model in python.
# # Copyright (C) 2022  Samuel Juhel
# #
# # This program is free software: you can redistribute it and/or modify
# # it under the terms of the GNU General Public License as published by
# # the Free Software Foundation, either version 3 of the License, or
# # (at your option) any later version.
# #
# # This program is distributed in the hope that it will be useful,
# # but WITHOUT ANY WARRANTY; without even the implied warranty of
# # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# # GNU General Public License for more details.
# #
# # You should have received a copy of the GNU General Public License
# # along with this program.  If not, see <https://www.gnu.org/licenses/>.

# import os
# import re
# import sys

# module_path = os.path.abspath(os.path.join('../'))
# if module_path not in sys.path:
#     sys.path.append(module_path)

# module_path = os.path.abspath(os.path.join('./'))
# if module_path not in sys.path:
#     sys.path.append(module_path)

# from boario.indicators import Indicators
# import pathlib
# import csv
# import logging
# from datetime import datetime
# import argparse

# parser = argparse.ArgumentParser(description='Aggregate an exio3 MRIO sectors')
# parser.add_argument('main_folder', type=str, help='A folder which subfolders are results of run')
# parser.add_argument('-o', "--output", type=str, help='The str path to save the pickled mrio to', nargs='?', default='../')
# args = parser.parse_args()
# logFormatter = logging.Formatter("%(asctime)s [%(levelname)-5.5s] %(name)s %(message)s", datefmt="%H:%M:%S")
# scriptLogger = logging.getLogger("indicators_batch")
# consoleHandler = logging.StreamHandler()
# consoleHandler.setFormatter(logFormatter)

# scriptLogger.addHandler(consoleHandler)
# scriptLogger.setLevel(logging.INFO)
# scriptLogger.propagate = False


# res = [{
#     "tot_fd_unmet": "unset",
#     "aff_fd_unmet": "unset",
#     "rebuild_durations": "unset",
#     "shortage_b": False,
#     "shortage_date_start": "unset",
#     "shortage_date_end": "unset",
#     "shortage_date_max": "unset",
#     "shortage_ind_max": "unset",
#     "shortage_ind_mean": "unset",
#     "10_first_shortages_(step,region,sector,stock_of)": "unset",
#     "prod_gain_tot": "unset",
#     "prod_lost_tot": "unset",
#     "prod_gain_unaff": "unset",
#     "prod_lost_unaff": "unset",
#     "region":"unset",
#     "q_dmg":"unset",
#     "gdp": "unset",
#     "type":"unset",
#     "psi": "unset",
#     "inv_tau": "unset",
#     "flood_int":"unset",
#     "n_timesteps":"unset",
#     "top_5_sector_loss":"unset",
#     "has_crashed":"unset"
# }]

# def batch(wd:pathlib.Path, out):
#     subdirs = [d for d in wd.iterdir() if d.is_dir()]
#     scriptLogger.info("Subdirs found : {}".format(subdirs))
#     for d in subdirs:
#         regex = re.compile(r"(?P<region>[A-Z]{2})_type_(?P<type>RoW|Full)_qdmg_(?P<sce>:raw_|int_)?(?P<gdp_dmg>(?:[\d_]+%?)|(?:max|min))_Psi_(?P<psi>(?:0_\d+)|1_0)_inv_tau_(?P<inv_tau>\d+)(?:_inv_time_(?:\d+))?")
#         #regex = re.compile(r"(?P<region>[A-Z]{2})_(?P<type>RoW|Full)_qdmg_(?:(?P<gdp_dmg>[\d.]+)|(?P<flood_int>[\d.]+%))_Psi_(?P<psi>0_\d+)_inv_tau_(?P<inv_tau>\d+)")
#         m = regex.match(d.name)
#         if m is None :
#             scriptLogger.warning("Directory {} didn't match regex".format(d.name))
#         else:
#             regex_res = m.groupdict()
#             region = regex_res['region']
#             sce_type = regex_res['sce']
#             if sce_type == "raw_":
#                 qdmg = regex_res['gdp_dmg']
#                 flood_centile = None
#             else:
#                 flood_centile = regex_res['gdp_dmg']
#                 qdmg = None
#             psi = regex_res['psi']
#             inv_tau = regex_res['inv_tau']
#             indic = Indicators.from_folder(d, d/"indexes.json")
#             indic.update_indicators()
#             res_a = indic.indicators.copy()
#             indic.write_indicators()
#             res_a['qdmg'] = qdmg
#             res_a['region'] = region
#             res_a['flood_centile'] = flood_centile
#             res_a['psi'] = psi
#             res_a['inv_tau'] = inv_tau
#             res_a['type'] = sce_type
#             res.append(res_a)

#     with (out/("indicators_"+datetime.now().strftime("%Y%m%d-%H%M")+".csv")).open('w') as f:
#         w = csv.DictWriter(f, res[0].keys())
#         w.writeheader()
#         w.writerows(res)

# if __name__ == '__main__':
#     scriptLogger.info('Starting Script')
#     args = parser.parse_args()
#     wd = pathlib.Path(args.main_folder).resolve()
#     if args.output == '../':
#         out = pathlib.Path(args.main_folder+'/'+args.output).resolve()
#     else:
#         out = pathlib.Path(args.output).resolve()
#     batch(wd, out)



"""
This type stub file was generated by pyright.
"""

import functools
import numpy as np
from typing import Callable
from pandas._typing import Scalar

@functools.lru_cache(maxsize=None)
def generate_numba_apply_func(func: Callable[..., Scalar], nopython: bool, nogil: bool, parallel: bool): # -> (values: ndarray[Unknown, Unknown], begin: ndarray[Unknown, Unknown], end: ndarray[Unknown, Unknown], minimum_periods: int, *args: Any) -> ndarray[Unknown, Unknown]:
    """
    Generate a numba jitted apply function specified by values from engine_kwargs.

    1. jit the user's function
    2. Return a rolling apply function with the jitted function inline

    Configurations specified in engine_kwargs apply to both the user's
    function _AND_ the rolling apply function.

    Parameters
    ----------
    func : function
        function to be applied to each window and will be JITed
    nopython : bool
        nopython to be passed into numba.jit
    nogil : bool
        nogil to be passed into numba.jit
    parallel : bool
        parallel to be passed into numba.jit

    Returns
    -------
    Numba function
    """
    ...

@functools.lru_cache(maxsize=None)
def generate_numba_ewm_func(nopython: bool, nogil: bool, parallel: bool, com: float, adjust: bool, ignore_na: bool, deltas: tuple, normalize: bool): # -> (values: ndarray[Unknown, Unknown], begin: ndarray[Unknown, Unknown], end: ndarray[Unknown, Unknown], minimum_periods: int) -> ndarray[Unknown, Unknown]:
    """
    Generate a numba jitted ewm mean or sum function specified by values
    from engine_kwargs.

    Parameters
    ----------
    nopython : bool
        nopython to be passed into numba.jit
    nogil : bool
        nogil to be passed into numba.jit
    parallel : bool
        parallel to be passed into numba.jit
    com : float
    adjust : bool
    ignore_na : bool
    deltas : tuple
    normalize : bool

    Returns
    -------
    Numba function
    """
    ...

@functools.lru_cache(maxsize=None)
def generate_numba_table_func(func: Callable[..., np.ndarray], nopython: bool, nogil: bool, parallel: bool): # -> (values: ndarray[Unknown, Unknown], begin: ndarray[Unknown, Unknown], end: ndarray[Unknown, Unknown], minimum_periods: int, *args: Any) -> NDArray[Any]:
    """
    Generate a numba jitted function to apply window calculations table-wise.

    Func will be passed a M window size x N number of columns array, and
    must return a 1 x N number of columns array. Func is intended to operate
    row-wise, but the result will be transposed for axis=1.

    1. jit the user's function
    2. Return a rolling apply function with the jitted function inline

    Parameters
    ----------
    func : function
        function to be applied to each window and will be JITed
    nopython : bool
        nopython to be passed into numba.jit
    nogil : bool
        nogil to be passed into numba.jit
    parallel : bool
        parallel to be passed into numba.jit

    Returns
    -------
    Numba function
    """
    ...

@functools.lru_cache(maxsize=None)
def generate_manual_numpy_nan_agg_with_axis(nan_func): # -> (table: Unknown) -> NDArray[float64]:
    ...

@functools.lru_cache(maxsize=None)
def generate_numba_ewm_table_func(nopython: bool, nogil: bool, parallel: bool, com: float, adjust: bool, ignore_na: bool, deltas: tuple, normalize: bool): # -> (values: ndarray[Unknown, Unknown], begin: ndarray[Unknown, Unknown], end: ndarray[Unknown, Unknown], minimum_periods: int) -> ndarray[Unknown, Unknown]:
    """
    Generate a numba jitted ewm mean or sum function applied table wise specified
    by values from engine_kwargs.

    Parameters
    ----------
    nopython : bool
        nopython to be passed into numba.jit
    nogil : bool
        nogil to be passed into numba.jit
    parallel : bool
        parallel to be passed into numba.jit
    com : float
    adjust : bool
    ignore_na : bool
    deltas : tuple
    normalize: bool

    Returns
    -------
    Numba function
    """
    ...




"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING

"""
Core eval alignment algorithms.
"""
if TYPE_CHECKING:
    ...
def align_terms(terms): # -> tuple[ABCSeries | ABCDataFrame, dict[str, Index]] | tuple[dtype[Any], None] | tuple[Any, None] | tuple[Unknown, Unknown]:
    """
    Align a set of terms.
    """
    ...

def reconstruct_object(typ, obj, axes, dtype): # -> PandasObject | Any | NDArray[Any] | ndarray[Unknown, Unknown]:
    """
    Reconstruct an object given its type, raw value, and possibly empty
    (None) axes.

    Parameters
    ----------
    typ : object
        A type
    obj : object
        The value to use in the type constructor
    axes : dict
        The axes to use to construct the resulting pandas object

    Returns
    -------
    ret : typ
        An object of type ``typ`` with the value `obj` and possible axes
        `axes`.
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
import pyarrow
from typing import Any, MutableMapping, TYPE_CHECKING
from pandas._libs import missing as libmissing
from pandas._libs.interval import Interval
from pandas._libs.properties import cache_readonly
from pandas._libs.tslibs import NaTType, Period, Timestamp, dtypes
from pandas._typing import DtypeObj, Ordered, npt, type_t
from pandas.core.dtypes.base import ExtensionDtype, register_extension_dtype
from datetime import tzinfo
from pandas import Categorical, Index
from pandas.core.arrays import BaseMaskedArray, DatetimeArray, IntervalArray, PandasArray, PeriodArray

"""
Define extension dtypes.
"""
if TYPE_CHECKING:
    ...
str_type = str
class PandasExtensionDtype(ExtensionDtype):
    """
    A np.dtype duck-typed class, suitable for holding a custom dtype.

    THIS IS NOT A REAL NUMPY DTYPE
    """
    type: Any
    kind: Any
    subdtype = ...
    str: str_type
    num = ...
    shape: tuple[int, ...] = ...
    itemsize = ...
    base: DtypeObj | None = ...
    isbuiltin = ...
    isnative = ...
    _cache_dtypes: dict[str_type, PandasExtensionDtype] = ...
    def __repr__(self) -> str_type:
        """
        Return a string representation for a particular object.
        """
        ...
    
    def __hash__(self) -> int:
        ...
    
    def __getstate__(self) -> dict[str_type, Any]:
        ...
    
    @classmethod
    def reset_cache(cls) -> None:
        """clear the cache"""
        ...
    


class CategoricalDtypeType(type):
    """
    the type of CategoricalDtype, this metaclass determines subclass ability
    """
    ...


@register_extension_dtype
class CategoricalDtype(PandasExtensionDtype, ExtensionDtype):
    """
    Type for categorical data with the categories and orderedness.

    Parameters
    ----------
    categories : sequence, optional
        Must be unique, and must not contain any nulls.
        The categories are stored in an Index,
        and if an index is provided the dtype of that index will be used.
    ordered : bool or None, default False
        Whether or not this categorical is treated as a ordered categorical.
        None can be used to maintain the ordered value of existing categoricals when
        used in operations that combine categoricals, e.g. astype, and will resolve to
        False if there is no existing ordered to maintain.

    Attributes
    ----------
    categories
    ordered

    Methods
    -------
    None

    See Also
    --------
    Categorical : Represent a categorical variable in classic R / S-plus fashion.

    Notes
    -----
    This class is useful for specifying the type of a ``Categorical``
    independent of the values. See :ref:`categorical.categoricaldtype`
    for more.

    Examples
    --------
    >>> t = pd.CategoricalDtype(categories=['b', 'a'], ordered=True)
    >>> pd.Series(['a', 'b', 'a', 'c'], dtype=t)
    0      a
    1      b
    2      a
    3    NaN
    dtype: category
    Categories (2, object): ['b' < 'a']

    An empty CategoricalDtype with a specific dtype can be created
    by providing an empty index. As follows,

    >>> pd.CategoricalDtype(pd.DatetimeIndex([])).categories.dtype
    dtype('<M8[ns]')
    """
    name = ...
    type: type[CategoricalDtypeType] = ...
    kind: str_type = ...
    str = ...
    base = ...
    _metadata = ...
    _cache_dtypes: dict[str_type, PandasExtensionDtype] = ...
    def __init__(self, categories=..., ordered: Ordered = ...) -> None:
        ...
    
    @classmethod
    def construct_from_string(cls, string: str_type) -> CategoricalDtype:
        """
        Construct a CategoricalDtype from a string.

        Parameters
        ----------
        string : str
            Must be the string "category" in order to be successfully constructed.

        Returns
        -------
        CategoricalDtype
            Instance of the dtype.

        Raises
        ------
        TypeError
            If a CategoricalDtype cannot be constructed from the input.
        """
        ...
    
    def __setstate__(self, state: MutableMapping[str_type, Any]) -> None:
        ...
    
    def __hash__(self) -> int:
        ...
    
    def __eq__(self, other: Any) -> bool:
        """
        Rules for CDT equality:
        1) Any CDT is equal to the string 'category'
        2) Any CDT is equal to itself
        3) Any CDT is equal to a CDT with categories=None regardless of ordered
        4) A CDT with ordered=True is only equal to another CDT with
           ordered=True and identical categories in the same order
        5) A CDT with ordered={False, None} is only equal to another CDT with
           ordered={False, None} and identical categories, but same order is
           not required. There is no distinction between False/None.
        6) Any other comparison returns False
        """
        ...
    
    def __repr__(self) -> str_type:
        ...
    
    @classmethod
    def construct_array_type(cls) -> type_t[Categorical]:
        """
        Return the array type associated with this dtype.

        Returns
        -------
        type
        """
        ...
    
    @staticmethod
    def validate_ordered(ordered: Ordered) -> None:
        """
        Validates that we have a valid ordered parameter. If
        it is not a boolean, a TypeError will be raised.

        Parameters
        ----------
        ordered : object
            The parameter to be verified.

        Raises
        ------
        TypeError
            If 'ordered' is not a boolean.
        """
        ...
    
    @staticmethod
    def validate_categories(categories, fastpath: bool = ...) -> Index:
        """
        Validates that we have good categories

        Parameters
        ----------
        categories : array-like
        fastpath : bool
            Whether to skip nan and uniqueness checks

        Returns
        -------
        categories : Index
        """
        ...
    
    def update_dtype(self, dtype: str_type | CategoricalDtype) -> CategoricalDtype:
        """
        Returns a CategoricalDtype with categories and ordered taken from dtype
        if specified, otherwise falling back to self if unspecified

        Parameters
        ----------
        dtype : CategoricalDtype

        Returns
        -------
        new_dtype : CategoricalDtype
        """
        ...
    
    @property
    def categories(self) -> Index:
        """
        An ``Index`` containing the unique categories allowed.
        """
        ...
    
    @property
    def ordered(self) -> Ordered:
        """
        Whether the categories have an ordered relationship.
        """
        ...
    


@register_extension_dtype
class DatetimeTZDtype(PandasExtensionDtype):
    """
    An ExtensionDtype for timezone-aware datetime data.

    **This is not an actual numpy dtype**, but a duck type.

    Parameters
    ----------
    unit : str, default "ns"
        The precision of the datetime data. Currently limited
        to ``"ns"``.
    tz : str, int, or datetime.tzinfo
        The timezone.

    Attributes
    ----------
    unit
    tz

    Methods
    -------
    None

    Raises
    ------
    pytz.UnknownTimeZoneError
        When the requested timezone cannot be found.

    Examples
    --------
    >>> pd.DatetimeTZDtype(tz='UTC')
    datetime64[ns, UTC]

    >>> pd.DatetimeTZDtype(tz='dateutil/US/Central')
    datetime64[ns, tzfile('/usr/share/zoneinfo/US/Central')]
    """
    type: type[Timestamp] = ...
    kind: str_type = ...
    num = ...
    base = ...
    _metadata = ...
    _match = ...
    _cache_dtypes: dict[str_type, PandasExtensionDtype] = ...
    @property
    def na_value(self) -> NaTType:
        ...
    
    @cache_readonly
    def str(self): # -> str:
        ...
    
    def __init__(self, unit: str_type | DatetimeTZDtype = ..., tz=...) -> None:
        ...
    
    @property
    def unit(self) -> str_type:
        """
        The precision of the datetime data.
        """
        ...
    
    @property
    def tz(self) -> tzinfo:
        """
        The timezone.
        """
        ...
    
    @classmethod
    def construct_array_type(cls) -> type_t[DatetimeArray]:
        """
        Return the array type associated with this dtype.

        Returns
        -------
        type
        """
        ...
    
    @classmethod
    def construct_from_string(cls, string: str_type) -> DatetimeTZDtype:
        """
        Construct a DatetimeTZDtype from a string.

        Parameters
        ----------
        string : str
            The string alias for this DatetimeTZDtype.
            Should be formatted like ``datetime64[ns, <tz>]``,
            where ``<tz>`` is the timezone name.

        Examples
        --------
        >>> DatetimeTZDtype.construct_from_string('datetime64[ns, UTC]')
        datetime64[ns, UTC]
        """
        ...
    
    def __str__(self) -> str_type:
        ...
    
    @property
    def name(self) -> str_type:
        """A string representation of the dtype."""
        ...
    
    def __hash__(self) -> int:
        ...
    
    def __eq__(self, other: Any) -> bool:
        ...
    
    def __setstate__(self, state) -> None:
        ...
    


@register_extension_dtype
class PeriodDtype(dtypes.PeriodDtypeBase, PandasExtensionDtype):
    """
    An ExtensionDtype for Period data.

    **This is not an actual numpy dtype**, but a duck type.

    Parameters
    ----------
    freq : str or DateOffset
        The frequency of this PeriodDtype.

    Attributes
    ----------
    freq

    Methods
    -------
    None

    Examples
    --------
    >>> pd.PeriodDtype(freq='D')
    period[D]

    >>> pd.PeriodDtype(freq=pd.offsets.MonthEnd())
    period[M]
    """
    type: type[Period] = ...
    kind: str_type = ...
    str = ...
    base = ...
    num = ...
    _metadata = ...
    _match = ...
    _cache_dtypes: dict[str_type, PandasExtensionDtype] = ...
    def __new__(cls, freq=...): # -> PeriodDtype | PandasExtensionDtype:
        """
        Parameters
        ----------
        freq : frequency
        """
        ...
    
    def __reduce__(self): # -> tuple[Type[Self@PeriodDtype], tuple[Unknown]]:
        ...
    
    @property
    def freq(self):
        """
        The frequency object of this PeriodDtype.
        """
        ...
    
    @classmethod
    def construct_from_string(cls, string: str_type) -> PeriodDtype:
        """
        Strict construction from a string, raise a TypeError if not
        possible
        """
        ...
    
    def __str__(self) -> str_type:
        ...
    
    @property
    def name(self) -> str_type:
        ...
    
    @property
    def na_value(self) -> NaTType:
        ...
    
    def __hash__(self) -> int:
        ...
    
    def __eq__(self, other: Any) -> bool:
        ...
    
    def __ne__(self, other: Any) -> bool:
        ...
    
    def __setstate__(self, state) -> None:
        ...
    
    @classmethod
    def is_dtype(cls, dtype: object) -> bool:
        """
        Return a boolean if we if the passed type is an actual dtype that we
        can match (via string or type)
        """
        ...
    
    @classmethod
    def construct_array_type(cls) -> type_t[PeriodArray]:
        """
        Return the array type associated with this dtype.

        Returns
        -------
        type
        """
        ...
    
    def __from_arrow__(self, array: pyarrow.Array | pyarrow.ChunkedArray) -> PeriodArray:
        """
        Construct PeriodArray from pyarrow Array/ChunkedArray.
        """
        ...
    


@register_extension_dtype
class IntervalDtype(PandasExtensionDtype):
    """
    An ExtensionDtype for Interval data.

    **This is not an actual numpy dtype**, but a duck type.

    Parameters
    ----------
    subtype : str, np.dtype
        The dtype of the Interval bounds.

    Attributes
    ----------
    subtype

    Methods
    -------
    None

    Examples
    --------
    >>> pd.IntervalDtype(subtype='int64', closed='both')
    interval[int64, both]
    """
    name = ...
    kind: str_type = ...
    str = ...
    base = ...
    num = ...
    _metadata = ...
    _match = ...
    _cache_dtypes: dict[str_type, PandasExtensionDtype] = ...
    def __new__(cls, subtype=..., closed: str_type | None = ...): # -> IntervalDtype | Self@IntervalDtype | PandasExtensionDtype:
        ...
    
    @property
    def closed(self):
        ...
    
    @property
    def subtype(self):
        """
        The dtype of the Interval bounds.
        """
        ...
    
    @classmethod
    def construct_array_type(cls) -> type[IntervalArray]:
        """
        Return the array type associated with this dtype.

        Returns
        -------
        type
        """
        ...
    
    @classmethod
    def construct_from_string(cls, string: str_type) -> IntervalDtype:
        """
        attempt to construct this type from a string, raise a TypeError
        if its not possible
        """
        ...
    
    @property
    def type(self) -> type[Interval]:
        ...
    
    def __str__(self) -> str_type:
        ...
    
    def __hash__(self) -> int:
        ...
    
    def __eq__(self, other: Any) -> bool:
        ...
    
    def __setstate__(self, state) -> None:
        ...
    
    @classmethod
    def is_dtype(cls, dtype: object) -> bool:
        """
        Return a boolean if we if the passed type is an actual dtype that we
        can match (via string or type)
        """
        ...
    
    def __from_arrow__(self, array: pyarrow.Array | pyarrow.ChunkedArray) -> IntervalArray:
        """
        Construct IntervalArray from pyarrow Array/ChunkedArray.
        """
        ...
    


class PandasDtype(ExtensionDtype):
    """
    A Pandas ExtensionDtype for NumPy dtypes.

    This is mostly for internal compatibility, and is not especially
    useful on its own.

    Parameters
    ----------
    dtype : object
        Object to be converted to a NumPy data type object.

    See Also
    --------
    numpy.dtype
    """
    _metadata = ...
    def __init__(self, dtype: npt.DTypeLike | PandasDtype | None) -> None:
        ...
    
    def __repr__(self) -> str:
        ...
    
    @property
    def numpy_dtype(self) -> np.dtype:
        """
        The NumPy dtype this PandasDtype wraps.
        """
        ...
    
    @property
    def name(self) -> str:
        """
        A bit-width name for this data-type.
        """
        ...
    
    @property
    def type(self) -> type[np.generic]:
        """
        The type object used to instantiate a scalar of this NumPy data-type.
        """
        ...
    
    @classmethod
    def construct_from_string(cls, string: str) -> PandasDtype:
        ...
    
    @classmethod
    def construct_array_type(cls) -> type_t[PandasArray]:
        """
        Return the array type associated with this dtype.

        Returns
        -------
        type
        """
        ...
    
    @property
    def kind(self) -> str:
        """
        A character code (one of 'biufcmMOSUV') identifying the general kind of data.
        """
        ...
    
    @property
    def itemsize(self) -> int:
        """
        The element size of this data-type object.
        """
        ...
    


class BaseMaskedDtype(ExtensionDtype):
    """
    Base class for dtypes for BaseMaskedArray subclasses.
    """
    name: str
    base = ...
    type: type
    @property
    def na_value(self) -> libmissing.NAType:
        ...
    
    @cache_readonly
    def numpy_dtype(self) -> np.dtype:
        """Return an instance of our numpy dtype"""
        ...
    
    @cache_readonly
    def kind(self) -> str:
        ...
    
    @cache_readonly
    def itemsize(self) -> int:
        """Return the number of bytes in this dtype"""
        ...
    
    @classmethod
    def construct_array_type(cls) -> type_t[BaseMaskedArray]:
        """
        Return the array type associated with this dtype.

        Returns
        -------
        type
        """
        ...
    
    @classmethod
    def from_numpy_dtype(cls, dtype: np.dtype) -> BaseMaskedDtype:
        """
        Construct the MaskedDtype corresponding to the given numpy dtype.
        """
        ...
    





import os
import re
import sys
import itertools
import boario.simulation as sim
from boario.indicators import Indicators
from boario.event import Event
import json
import pathlib
import numpy as np
import pandas as pd

base_time = 1

# We instantiate a dictionary with the parameters
# (it is also possible to use a json file)
params = {
    # The name of the working directory to use (relative to current wd)
    "input_dir": "/home/sjuhel/Nextcloud/Thesis/Workbench/Data/BoARIO-inputs/",
    "output_dir": "/home/sjuhel/Nextcloud/Thesis/Workbench/Data/BoARIO-testing/dev-testing-new-shocks",
    # The directory to use to store results (relative to storage_dir)
    # i.e. here, the model will look for files in ~/boario/storage/ and
    # store results in ~/boario/storage/results/
    "results_storage": "results",
    # This tells the model to register the evolution of the stocks
    # of every industry (the file can be quite large (2Gbytes+ for
    # a 365 days simulation with exiobase))
    "register_stocks": False,
    "psi_param": 0.90,
    "order_type": "alt",
    "model_type": "ARIOPsi",
    "temporal_units_by_step": 1,
    "year_to_temporal_unit_factor": 365,
    "inventory_restoration_tau": 60,
    "alpha_base": 1.0,
    "alpha_max": 1.25,
    "alpha_tau": 365,
    "rebuild_tau": 60,
    "n_temporal_units_to_sim": 700,
    "impacted_region_base_production_toward_rebuilding": 0.0,
    "row_base_production_toward_rebuilding": 0.0,
   "mrio_params_file": "/home/sjuhel/Nextcloud/Thesis/Workbench/Data/BoARIO-inputs/builded-data/params/exiobase3_7_sectors_params.json"
}

#with open("/home/sjuhel/Nextcloud/Thesis/Workbench/Data/BoARIO-inputs/exps/experience-ciclad-parameters-explore-1/exiobase3_74_sectors_event_template.json") as f:
    #event = json.load(f)

with open("/home/sjuhel/Nextcloud/Thesis/Workbench/Data/BoARIO-inputs/builded-data/params/exiobase3_7_sectors_event_params.json") as f:
    event = json.load(f)

event["occur"] = 2
event["kapital_damage"] = 10000000000
event["q_dmg"] = event["kapital_damage"]
event["duration"] = 10
event["aff_sectors"] = ["Mining", "Manufacture"]
event["shock_type"] = "kapital_destroyed_rebuild"

event['dmg_distrib_regions'] = event['dmg_regional_distrib']
event['dmg_distrib_sectors_type'] = event['dmg_sectoral_distrib_type']
event['dmg_distrib_sectors'] = event['dmg_distrib_sectors_type']

event2 = event.copy()
event2["occur"] = 5
event2["aff_regions"] = ["DE"]

# We load the mrio table from a pickle file (created with the help of the
# pymrio module, more on that in the doc)
mrio_path = pathlib.Path("/home/sjuhel/Nextcloud/Thesis/Workbench/Data/BoARIO-inputs/builded-data/mrios/Exiobase/exiobase3_2011_7_sectors.pkl")

simul = sim.Simulation(params, mrio_path, modeltype="ARIOPsi")
simul.read_events_from_list([event])
simul.loop()
indic = Indicators.from_folder(
                           pathlib.Path(params['output_dir'])/params['results_storage'],
                           indexes_file=pathlib.Path(params['output_dir'])/params['results_storage']/"indexes.json",
                            include_crash=True)
indic.update_indicators()
indic.write_indicators()



"""
This type stub file was generated by pyright.
"""

__all__ = ["array", "ArrowDtype", "bdate_range", "BooleanDtype", "Categorical", "CategoricalDtype", "CategoricalIndex", "DataFrame", "DateOffset", "date_range", "DatetimeIndex", "DatetimeTZDtype", "factorize", "Flags", "Float32Dtype", "Float64Dtype", "Float64Index", "Grouper", "Index", "IndexSlice", "Int16Dtype", "Int32Dtype", "Int64Dtype", "Int64Index", "Int8Dtype", "Interval", "IntervalDtype", "IntervalIndex", "interval_range", "isna", "isnull", "MultiIndex", "NA", "NamedAgg", "NaT", "notna", "notnull", "NumericIndex", "Period", "PeriodDtype", "PeriodIndex", "period_range", "RangeIndex", "Series", "set_eng_float_format", "StringDtype", "Timedelta", "TimedeltaIndex", "timedelta_range", "Timestamp", "to_datetime", "to_numeric", "to_timedelta", "UInt16Dtype", "UInt32Dtype", "UInt64Dtype", "UInt64Index", "UInt8Dtype", "unique", "value_counts"]



"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING
from pandas._typing import FilePath, ReadBuffer, Scalar, StorageOptions
from pandas.util._decorators import doc
from pandas.core.shared_docs import _shared_docs
from pandas.io.excel._base import BaseExcelReader
from pandas._libs.tslibs.nattype import NaTType

if TYPE_CHECKING:
    ...
@doc(storage_options=_shared_docs["storage_options"])
class ODFReader(BaseExcelReader):
    """
    Read tables out of OpenDocument formatted files.

    Parameters
    ----------
    filepath_or_buffer : str, path to be parsed or
        an open readable stream.
    {storage_options}
    """
    def __init__(self, filepath_or_buffer: FilePath | ReadBuffer[bytes], storage_options: StorageOptions = ...) -> None:
        ...
    
    def load_workbook(self, filepath_or_buffer: FilePath | ReadBuffer[bytes]):
        ...
    
    @property
    def empty_value(self) -> str:
        """Property for compat with other readers."""
        ...
    
    @property
    def sheet_names(self) -> list[str]:
        """Return a list of sheet names present in the document"""
        ...
    
    def get_sheet_by_index(self, index: int):
        ...
    
    def get_sheet_by_name(self, name: str):
        ...
    
    def get_sheet_data(self, sheet, convert_float: bool, file_rows_needed: int | None = ...) -> list[list[Scalar | NaTType]]:
        """
        Parse an ODF Table into a list of lists
        """
        ...
    





"""
This type stub file was generated by pyright.
"""

from typing import Callable
from pandas._typing import F

"""
Boilerplate functions used in defining binary operations.
"""
def unpack_zerodim_and_defer(name: str) -> Callable[[F], F]:
    """
    Boilerplate for pandas conventions in arithmetic and comparison methods.

    Parameters
    ----------
    name : str

    Returns
    -------
    decorator
    """
    ...

def get_op_result_name(left, right): # -> None:
    """
    Find the appropriate name to pin to an operation result.  This result
    should always be either an Index or a Series.

    Parameters
    ----------
    left : {Series, Index}
    right : object

    Returns
    -------
    name : object
        Usually a string
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._typing import npt

"""This module is designed for community supported date conversion functions"""
def parse_date_time(date_col, time_col) -> npt.NDArray[np.object_]:
    """
    Parse columns with dates and times into a single datetime column.

    .. deprecated:: 1.2
    """
    ...

def parse_date_fields(year_col, month_col, day_col) -> npt.NDArray[np.object_]:
    """
    Parse columns with years, months and days into a single date column.

    .. deprecated:: 1.2
    """
    ...

def parse_all_fields(year_col, month_col, day_col, hour_col, minute_col, second_col) -> npt.NDArray[np.object_]:
    """
    Parse columns with datetime information into a single datetime column.

    .. deprecated:: 1.2
    """
    ...

def generic_parser(parse_func, *cols) -> np.ndarray:
    """
    Use dateparser to parse columns with data information into a single datetime column.

    .. deprecated:: 1.2
    """
    ...




"""
This type stub file was generated by pyright.
"""

from pandas.io.json._json import dumps, loads, read_json, to_json
from pandas.io.json._normalize import _json_normalize, json_normalize
from pandas.io.json._table_schema import build_table_schema

__all__ = ["dumps", "loads", "read_json", "to_json", "_json_normalize", "json_normalize", "build_table_schema"]



"""
This type stub file was generated by pyright.
"""

from typing import Hashable, Sequence
from pandas._typing import FilePath, ReadBuffer, StorageOptions, WriteBuffer
from pandas.util._decorators import doc
from pandas.core.api import DataFrame
from pandas.core.shared_docs import _shared_docs

""" feather-format compat """
@doc(storage_options=_shared_docs["storage_options"])
def to_feather(df: DataFrame, path: FilePath | WriteBuffer[bytes], storage_options: StorageOptions = ..., **kwargs) -> None:
    """
    Write a DataFrame to the binary Feather format.

    Parameters
    ----------
    df : DataFrame
    path : str, path object, or file-like object
    {storage_options}

        .. versionadded:: 1.2.0

    **kwargs :
        Additional keywords passed to `pyarrow.feather.write_feather`.

        .. versionadded:: 1.1.0
    """
    ...

@doc(storage_options=_shared_docs["storage_options"])
def read_feather(path: FilePath | ReadBuffer[bytes], columns: Sequence[Hashable] | None = ..., use_threads: bool = ..., storage_options: StorageOptions = ...):
    """
    Load a feather-format object from the file path.

    Parameters
    ----------
    path : str, path object, or file-like object
        String, path object (implementing ``os.PathLike[str]``), or file-like
        object implementing a binary ``read()`` function. The string could be a URL.
        Valid URL schemes include http, ftp, s3, and file. For file URLs, a host is
        expected. A local file could be: ``file://localhost/path/to/table.feather``.
    columns : sequence, default None
        If not provided, all columns are read.
    use_threads : bool, default True
        Whether to parallelize reading using multiple threads.
    {storage_options}

        .. versionadded:: 1.2.0

    Returns
    -------
    type of object stored in file
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np
from typing import Hashable, Iterable, Iterator, TYPE_CHECKING
from pandas._typing import ArrayLike, npt
from pandas import DataFrame, Index, MultiIndex, Series

"""
data hash pandas / numpy objects
"""
if TYPE_CHECKING:
    ...
_default_hash_key = ...
def combine_hash_arrays(arrays: Iterator[np.ndarray], num_items: int) -> npt.NDArray[np.uint64]:
    """
    Parameters
    ----------
    arrays : Iterator[np.ndarray]
    num_items : int

    Returns
    -------
    np.ndarray[uint64]

    Should be the same as CPython's tupleobject.c
    """
    ...

def hash_pandas_object(obj: Index | DataFrame | Series, index: bool = ..., encoding: str = ..., hash_key: str | None = ..., categorize: bool = ...) -> Series:
    """
    Return a data hash of the Index/Series/DataFrame.

    Parameters
    ----------
    obj : Index, Series, or DataFrame
    index : bool, default True
        Include the index in the hash (if Series/DataFrame).
    encoding : str, default 'utf8'
        Encoding for data & key when strings.
    hash_key : str, default _default_hash_key
        Hash_key for string key to encode.
    categorize : bool, default True
        Whether to first categorize object arrays before hashing. This is more
        efficient when the array contains duplicate values.

    Returns
    -------
    Series of uint64, same length as the object
    """
    ...

def hash_tuples(vals: MultiIndex | Iterable[tuple[Hashable, ...]], encoding: str = ..., hash_key: str = ...) -> npt.NDArray[np.uint64]:
    """
    Hash an MultiIndex / listlike-of-tuples efficiently.

    Parameters
    ----------
    vals : MultiIndex or listlike-of-tuples
    encoding : str, default 'utf8'
    hash_key : str, default _default_hash_key

    Returns
    -------
    ndarray[np.uint64] of hashed values
    """
    ...

def hash_array(vals: ArrayLike, encoding: str = ..., hash_key: str = ..., categorize: bool = ...) -> npt.NDArray[np.uint64]:
    """
    Given a 1d array, return an array of deterministic integers.

    Parameters
    ----------
    vals : ndarray or ExtensionArray
    encoding : str, default 'utf8'
        Encoding for data & key when strings.
    hash_key : str, default _default_hash_key
        Hash_key for string key to encode.
    categorize : bool, default True
        Whether to first categorize object arrays before hashing. This is more
        efficient when the array contains duplicate values.

    Returns
    -------
    ndarray[np.uint64, ndim=1]
        Hashed values, same length as the vals.
    """
    ...




"""
This type stub file was generated by pyright.
"""

from pandas.core.indexers.utils import check_array_indexer, check_key_length, check_setitem_lengths, deprecate_ndim_indexing, is_empty_indexer, is_list_like_indexer, is_scalar_indexer, is_valid_positional_slice, length_of_indexer, maybe_convert_indices, unpack_1tuple, unpack_tuple_and_ellipses, validate_indices

__all__ = ["is_valid_positional_slice", "is_list_like_indexer", "is_scalar_indexer", "is_empty_indexer", "check_setitem_lengths", "validate_indices", "maybe_convert_indices", "length_of_indexer", "deprecate_ndim_indexing", "unpack_1tuple", "check_key_length", "check_array_indexer", "unpack_tuple_and_ellipses"]



"""
This type stub file was generated by pyright.
"""

import numpy as np
from pandas._typing import npt

class OutOfBoundsDatetime(ValueError):
    ...


class OutOfBoundsTimedelta(ValueError):
    ...


def py_get_unit_from_dtype(dtype: np.dtype):
    ...

def py_td64_to_tdstruct(td64: int, unit: int) -> dict:
    ...

def astype_overflowsafe(arr: np.ndarray, dtype: np.dtype, copy: bool = ..., round_ok: bool = ...) -> np.ndarray:
    ...

def is_unitless(dtype: np.dtype) -> bool:
    ...

def compare_mismatched_resolutions(left: np.ndarray, right: np.ndarray, op) -> npt.NDArray[np.bool_]:
    ...




"""
This type stub file was generated by pyright.
"""

from datetime import datetime
from typing import List, TYPE_CHECKING, Tuple, TypedDict, Union, overload
from pandas._libs.tslibs import Timestamp
from pandas._typing import AnyArrayLike, ArrayLike, DateTimeErrorChoices
from pandas.core.indexes.base import Index
from pandas.core.indexes.datetimes import DatetimeIndex
from pandas._libs.tslibs.nattype import NaTType
from pandas import Series

if TYPE_CHECKING:
    ...
ArrayConvertible = Union[List, Tuple, AnyArrayLike]
Scalar = Union[float, str]
DatetimeScalar = Union[Scalar, datetime]
DatetimeScalarOrArrayConvertible = Union[DatetimeScalar, ArrayConvertible]
DatetimeDictArg = Union[List[Scalar], Tuple[Scalar, ...], AnyArrayLike]
class YearMonthDayDict(TypedDict, total=True):
    year: DatetimeDictArg
    month: DatetimeDictArg
    day: DatetimeDictArg
    ...


class FulldatetimeDict(YearMonthDayDict, total=False):
    hour: DatetimeDictArg
    hours: DatetimeDictArg
    minute: DatetimeDictArg
    minutes: DatetimeDictArg
    second: DatetimeDictArg
    seconds: DatetimeDictArg
    ms: DatetimeDictArg
    us: DatetimeDictArg
    ns: DatetimeDictArg
    ...


DictConvertible = Union[FulldatetimeDict, "DataFrame"]
start_caching_at = ...
def should_cache(arg: ArrayConvertible, unique_share: float = ..., check_count: int | None = ...) -> bool:
    """
    Decides whether to do caching.

    If the percent of unique elements among `check_count` elements less
    than `unique_share * 100` then we can do caching.

    Parameters
    ----------
    arg: listlike, tuple, 1-d array, Series
    unique_share: float, default=0.7, optional
        0 < unique_share < 1
    check_count: int, optional
        0 <= check_count <= len(arg)

    Returns
    -------
    do_caching: bool

    Notes
    -----
    By default for a sequence of less than 50 items in size, we don't do
    caching; for the number of elements less than 5000, we take ten percent of
    all elements to check for a uniqueness share; if the sequence size is more
    than 5000, then we check only the first 500 elements.
    All constants were chosen empirically by.
    """
    ...

@overload
def to_datetime(arg: DatetimeScalar, errors: DateTimeErrorChoices = ..., dayfirst: bool = ..., yearfirst: bool = ..., utc: bool | None = ..., format: str | None = ..., exact: bool = ..., unit: str | None = ..., infer_datetime_format: bool = ..., origin=..., cache: bool = ...) -> Timestamp:
    ...

@overload
def to_datetime(arg: Series | DictConvertible, errors: DateTimeErrorChoices = ..., dayfirst: bool = ..., yearfirst: bool = ..., utc: bool | None = ..., format: str | None = ..., exact: bool = ..., unit: str | None = ..., infer_datetime_format: bool = ..., origin=..., cache: bool = ...) -> Series:
    ...

@overload
def to_datetime(arg: list | tuple | Index | ArrayLike, errors: DateTimeErrorChoices = ..., dayfirst: bool = ..., yearfirst: bool = ..., utc: bool | None = ..., format: str | None = ..., exact: bool = ..., unit: str | None = ..., infer_datetime_format: bool = ..., origin=..., cache: bool = ...) -> DatetimeIndex:
    ...

def to_datetime(arg: DatetimeScalarOrArrayConvertible | DictConvertible, errors: DateTimeErrorChoices = ..., dayfirst: bool = ..., yearfirst: bool = ..., utc: bool | None = ..., format: str | None = ..., exact: bool = ..., unit: str | None = ..., infer_datetime_format: bool = ..., origin=..., cache: bool = ...) -> DatetimeIndex | Series | DatetimeScalar | NaTType | None:
    """
    Convert argument to datetime.

    This function converts a scalar, array-like, :class:`Series` or
    :class:`DataFrame`/dict-like to a pandas datetime object.

    Parameters
    ----------
    arg : int, float, str, datetime, list, tuple, 1-d array, Series, DataFrame/dict-like
        The object to convert to a datetime. If a :class:`DataFrame` is provided, the
        method expects minimally the following columns: :const:`"year"`,
        :const:`"month"`, :const:`"day"`.
    errors : {'ignore', 'raise', 'coerce'}, default 'raise'
        - If :const:`'raise'`, then invalid parsing will raise an exception.
        - If :const:`'coerce'`, then invalid parsing will be set as :const:`NaT`.
        - If :const:`'ignore'`, then invalid parsing will return the input.
    dayfirst : bool, default False
        Specify a date parse order if `arg` is str or is list-like.
        If :const:`True`, parses dates with the day first, e.g. :const:`"10/11/12"`
        is parsed as :const:`2012-11-10`.

        .. warning::

            ``dayfirst=True`` is not strict, but will prefer to parse
            with day first. If a delimited date string cannot be parsed in
            accordance with the given `dayfirst` option, e.g.
            ``to_datetime(['31-12-2021'])``, then a warning will be shown.

    yearfirst : bool, default False
        Specify a date parse order if `arg` is str or is list-like.

        - If :const:`True` parses dates with the year first, e.g.
          :const:`"10/11/12"` is parsed as :const:`2010-11-12`.
        - If both `dayfirst` and `yearfirst` are :const:`True`, `yearfirst` is
          preceded (same as :mod:`dateutil`).

        .. warning::

            ``yearfirst=True`` is not strict, but will prefer to parse
            with year first.

    utc : bool, default None
        Control timezone-related parsing, localization and conversion.

        - If :const:`True`, the function *always* returns a timezone-aware
          UTC-localized :class:`Timestamp`, :class:`Series` or
          :class:`DatetimeIndex`. To do this, timezone-naive inputs are
          *localized* as UTC, while timezone-aware inputs are *converted* to UTC.

        - If :const:`False` (default), inputs will not be coerced to UTC.
          Timezone-naive inputs will remain naive, while timezone-aware ones
          will keep their time offsets. Limitations exist for mixed
          offsets (typically, daylight savings), see :ref:`Examples
          <to_datetime_tz_examples>` section for details.

        See also: pandas general documentation about `timezone conversion and
        localization
        <https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html
        #time-zone-handling>`_.

    format : str, default None
        The strftime to parse time, e.g. :const:`"%d/%m/%Y"`. Note that
        :const:`"%f"` will parse all the way up to nanoseconds. See
        `strftime documentation
        <https://docs.python.org/3/library/datetime.html
        #strftime-and-strptime-behavior>`_ for more information on choices.
    exact : bool, default True
        Control how `format` is used:

        - If :const:`True`, require an exact `format` match.
        - If :const:`False`, allow the `format` to match anywhere in the target
          string.

    unit : str, default 'ns'
        The unit of the arg (D,s,ms,us,ns) denote the unit, which is an
        integer or float number. This will be based off the origin.
        Example, with ``unit='ms'`` and ``origin='unix'``, this would calculate
        the number of milliseconds to the unix epoch start.
    infer_datetime_format : bool, default False
        If :const:`True` and no `format` is given, attempt to infer the format
        of the datetime strings based on the first non-NaN element,
        and if it can be inferred, switch to a faster method of parsing them.
        In some cases this can increase the parsing speed by ~5-10x.
    origin : scalar, default 'unix'
        Define the reference date. The numeric values would be parsed as number
        of units (defined by `unit`) since this reference date.

        - If :const:`'unix'` (or POSIX) time; origin is set to 1970-01-01.
        - If :const:`'julian'`, unit must be :const:`'D'`, and origin is set to
          beginning of Julian Calendar. Julian day number :const:`0` is assigned
          to the day starting at noon on January 1, 4713 BC.
        - If Timestamp convertible, origin is set to Timestamp identified by
          origin.
    cache : bool, default True
        If :const:`True`, use a cache of unique, converted dates to apply the
        datetime conversion. May produce significant speed-up when parsing
        duplicate date strings, especially ones with timezone offsets. The cache
        is only used when there are at least 50 values. The presence of
        out-of-bounds values will render the cache unusable and may slow down
        parsing.

        .. versionchanged:: 0.25.0
            changed default value from :const:`False` to :const:`True`.

    Returns
    -------
    datetime
        If parsing succeeded.
        Return type depends on input (types in parenthesis correspond to
        fallback in case of unsuccessful timezone or out-of-range timestamp
        parsing):

        - scalar: :class:`Timestamp` (or :class:`datetime.datetime`)
        - array-like: :class:`DatetimeIndex` (or :class:`Series` with
          :class:`object` dtype containing :class:`datetime.datetime`)
        - Series: :class:`Series` of :class:`datetime64` dtype (or
          :class:`Series` of :class:`object` dtype containing
          :class:`datetime.datetime`)
        - DataFrame: :class:`Series` of :class:`datetime64` dtype (or
          :class:`Series` of :class:`object` dtype containing
          :class:`datetime.datetime`)

    Raises
    ------
    ParserError
        When parsing a date from string fails.
    ValueError
        When another datetime conversion error happens. For example when one
        of 'year', 'month', day' columns is missing in a :class:`DataFrame`, or
        when a Timezone-aware :class:`datetime.datetime` is found in an array-like
        of mixed time offsets, and ``utc=False``.

    See Also
    --------
    DataFrame.astype : Cast argument to a specified dtype.
    to_timedelta : Convert argument to timedelta.
    convert_dtypes : Convert dtypes.

    Notes
    -----

    Many input types are supported, and lead to different output types:

    - **scalars** can be int, float, str, datetime object (from stdlib :mod:`datetime`
      module or :mod:`numpy`). They are converted to :class:`Timestamp` when
      possible, otherwise they are converted to :class:`datetime.datetime`.
      None/NaN/null scalars are converted to :const:`NaT`.

    - **array-like** can contain int, float, str, datetime objects. They are
      converted to :class:`DatetimeIndex` when possible, otherwise they are
      converted to :class:`Index` with :class:`object` dtype, containing
      :class:`datetime.datetime`. None/NaN/null entries are converted to
      :const:`NaT` in both cases.

    - **Series** are converted to :class:`Series` with :class:`datetime64`
      dtype when possible, otherwise they are converted to :class:`Series` with
      :class:`object` dtype, containing :class:`datetime.datetime`. None/NaN/null
      entries are converted to :const:`NaT` in both cases.

    - **DataFrame/dict-like** are converted to :class:`Series` with
      :class:`datetime64` dtype. For each row a datetime is created from assembling
      the various dataframe columns. Column keys can be common abbreviations
      like [‘year’, ‘month’, ‘day’, ‘minute’, ‘second’, ‘ms’, ‘us’, ‘ns’]) or
      plurals of the same.

    The following causes are responsible for :class:`datetime.datetime` objects
    being returned (possibly inside an :class:`Index` or a :class:`Series` with
    :class:`object` dtype) instead of a proper pandas designated type
    (:class:`Timestamp`, :class:`DatetimeIndex` or :class:`Series`
    with :class:`datetime64` dtype):

    - when any input element is before :const:`Timestamp.min` or after
      :const:`Timestamp.max`, see `timestamp limitations
      <https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html
      #timeseries-timestamp-limits>`_.

    - when ``utc=False`` (default) and the input is an array-like or
      :class:`Series` containing mixed naive/aware datetime, or aware with mixed
      time offsets. Note that this happens in the (quite frequent) situation when
      the timezone has a daylight savings policy. In that case you may wish to
      use ``utc=True``.

    Examples
    --------

    **Handling various input formats**

    Assembling a datetime from multiple columns of a :class:`DataFrame`. The keys
    can be common abbreviations like ['year', 'month', 'day', 'minute', 'second',
    'ms', 'us', 'ns']) or plurals of the same

    >>> df = pd.DataFrame({'year': [2015, 2016],
    ...                    'month': [2, 3],
    ...                    'day': [4, 5]})
    >>> pd.to_datetime(df)
    0   2015-02-04
    1   2016-03-05
    dtype: datetime64[ns]

    Passing ``infer_datetime_format=True`` can often-times speedup a parsing
    if its not an ISO8601 format exactly, but in a regular format.

    >>> s = pd.Series(['3/11/2000', '3/12/2000', '3/13/2000'] * 1000)
    >>> s.head()
    0    3/11/2000
    1    3/12/2000
    2    3/13/2000
    3    3/11/2000
    4    3/12/2000
    dtype: object

    >>> %timeit pd.to_datetime(s, infer_datetime_format=True)  # doctest: +SKIP
    100 loops, best of 3: 10.4 ms per loop

    >>> %timeit pd.to_datetime(s, infer_datetime_format=False)  # doctest: +SKIP
    1 loop, best of 3: 471 ms per loop

    Using a unix epoch time

    >>> pd.to_datetime(1490195805, unit='s')
    Timestamp('2017-03-22 15:16:45')
    >>> pd.to_datetime(1490195805433502912, unit='ns')
    Timestamp('2017-03-22 15:16:45.433502912')

    .. warning:: For float arg, precision rounding might happen. To prevent
        unexpected behavior use a fixed-width exact type.

    Using a non-unix epoch origin

    >>> pd.to_datetime([1, 2, 3], unit='D',
    ...                origin=pd.Timestamp('1960-01-01'))
    DatetimeIndex(['1960-01-02', '1960-01-03', '1960-01-04'],
                  dtype='datetime64[ns]', freq=None)

    **Non-convertible date/times**

    If a date does not meet the `timestamp limitations
    <https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html
    #timeseries-timestamp-limits>`_, passing ``errors='ignore'``
    will return the original input instead of raising any exception.

    Passing ``errors='coerce'`` will force an out-of-bounds date to :const:`NaT`,
    in addition to forcing non-dates (or non-parseable dates) to :const:`NaT`.

    >>> pd.to_datetime('13000101', format='%Y%m%d', errors='ignore')
    datetime.datetime(1300, 1, 1, 0, 0)
    >>> pd.to_datetime('13000101', format='%Y%m%d', errors='coerce')
    NaT

    .. _to_datetime_tz_examples:

    **Timezones and time offsets**

    The default behaviour (``utc=False``) is as follows:

    - Timezone-naive inputs are converted to timezone-naive :class:`DatetimeIndex`:

    >>> pd.to_datetime(['2018-10-26 12:00', '2018-10-26 13:00:15'])
    DatetimeIndex(['2018-10-26 12:00:00', '2018-10-26 13:00:15'],
                  dtype='datetime64[ns]', freq=None)

    - Timezone-aware inputs *with constant time offset* are converted to
      timezone-aware :class:`DatetimeIndex`:

    >>> pd.to_datetime(['2018-10-26 12:00 -0500', '2018-10-26 13:00 -0500'])
    DatetimeIndex(['2018-10-26 12:00:00-05:00', '2018-10-26 13:00:00-05:00'],
                  dtype='datetime64[ns, pytz.FixedOffset(-300)]', freq=None)

    - However, timezone-aware inputs *with mixed time offsets* (for example
      issued from a timezone with daylight savings, such as Europe/Paris)
      are **not successfully converted** to a :class:`DatetimeIndex`. Instead a
      simple :class:`Index` containing :class:`datetime.datetime` objects is
      returned:

    >>> pd.to_datetime(['2020-10-25 02:00 +0200', '2020-10-25 04:00 +0100'])
    Index([2020-10-25 02:00:00+02:00, 2020-10-25 04:00:00+01:00],
          dtype='object')

    - A mix of timezone-aware and timezone-naive inputs is converted to
      a timezone-aware :class:`DatetimeIndex` if the offsets of the timezone-aware
      are constant:

    >>> from datetime import datetime
    >>> pd.to_datetime(["2020-01-01 01:00 -01:00", datetime(2020, 1, 1, 3, 0)])
    DatetimeIndex(['2020-01-01 01:00:00-01:00', '2020-01-01 02:00:00-01:00'],
                  dtype='datetime64[ns, pytz.FixedOffset(-60)]', freq=None)

    |

    Setting ``utc=True`` solves most of the above issues:

    - Timezone-naive inputs are *localized* as UTC

    >>> pd.to_datetime(['2018-10-26 12:00', '2018-10-26 13:00'], utc=True)
    DatetimeIndex(['2018-10-26 12:00:00+00:00', '2018-10-26 13:00:00+00:00'],
                  dtype='datetime64[ns, UTC]', freq=None)

    - Timezone-aware inputs are *converted* to UTC (the output represents the
      exact same datetime, but viewed from the UTC time offset `+00:00`).

    >>> pd.to_datetime(['2018-10-26 12:00 -0530', '2018-10-26 12:00 -0500'],
    ...                utc=True)
    DatetimeIndex(['2018-10-26 17:30:00+00:00', '2018-10-26 17:00:00+00:00'],
                  dtype='datetime64[ns, UTC]', freq=None)

    - Inputs can contain both naive and aware, string or datetime, the above
      rules still apply

    >>> from datetime import timezone, timedelta
    >>> pd.to_datetime(['2018-10-26 12:00', '2018-10-26 12:00 -0530',
    ...                datetime(2020, 1, 1, 18),
    ...                datetime(2020, 1, 1, 18,
    ...                tzinfo=timezone(-timedelta(hours=1)))],
    ...                utc=True)
    DatetimeIndex(['2018-10-26 12:00:00+00:00', '2018-10-26 17:30:00+00:00',
                   '2020-01-01 18:00:00+00:00', '2020-01-01 19:00:00+00:00'],
                  dtype='datetime64[ns, UTC]', freq=None)
    """
    ...

_unit_map = ...
def to_time(arg, format=..., infer_time_format=..., errors=...): # -> time | Series | NDArray[Any] | list[time | None] | Any | None:
    ...

__all__ = ["DateParseError", "should_cache", "to_datetime", "to_time"]



# BoARIO : The Adaptative Regional Input Output model in python.
# Copyright (C) 2022  Samuel Juhel
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from boario.event import Event
from collections.abc import Iterable
import pymrio

def flatten(l):
    for el in l:
        if isinstance(el, Iterable) and not isinstance(el, (str, bytes)):
            yield from flatten(el)
        else:
            yield el

def lexico_reindex(mrio: pymrio.IOSystem) -> pymrio.IOSystem:
    """Reindex IOSystem lexicographicaly

    Sort indexes and columns of the dataframe of a ``pymrio`` `IOSystem <https://pymrio.readthedocs.io/en/latest/intro.html>` by
    lexical order.

    Parameters
    ----------
    mrio : pymrio.IOSystem
        The IOSystem to sort

    Returns
    -------
    pymrio.IOSystem
        The sorted IOSystem


    """
    if mrio.Z is None:
        raise ValueError("Given mrio has no Z attribute set")
    mrio.Z = mrio.Z.reindex(sorted(mrio.Z.index), axis=0)
    mrio.Z = mrio.Z.reindex(sorted(mrio.Z.columns), axis=1)
    if mrio.Y is None:
        raise ValueError("Given mrio has no Z attribute set")
    mrio.Y = mrio.Y.reindex(sorted(mrio.Y.index), axis=0)
    mrio.Y = mrio.Y.reindex(sorted(mrio.Y.columns), axis=1)
    if mrio.x is None:
        raise ValueError("Given mrio has no Z attribute set")
    mrio.x = mrio.x.reindex(sorted(mrio.x.index), axis=0)
    if mrio.A is None:
        raise ValueError("Given mrio has no Z attribute set")
    mrio.A = mrio.A.reindex(sorted(mrio.A.index), axis=0)
    mrio.A = mrio.A.reindex(sorted(mrio.A.columns), axis=1)

    return mrio



"""
This type stub file was generated by pyright.
"""

__all__ = ["is_array_like", "is_bool", "is_bool_dtype", "is_categorical", "is_categorical_dtype", "is_complex", "is_complex_dtype", "is_datetime64_any_dtype", "is_datetime64_dtype", "is_datetime64_ns_dtype", "is_datetime64tz_dtype", "is_dict_like", "is_dtype_equal", "is_extension_array_dtype", "is_extension_type", "is_file_like", "is_float", "is_float_dtype", "is_hashable", "is_int64_dtype", "is_integer", "is_integer_dtype", "is_interval", "is_interval_dtype", "is_iterator", "is_list_like", "is_named_tuple", "is_number", "is_numeric_dtype", "is_object_dtype", "is_period_dtype", "is_re", "is_re_compilable", "is_scalar", "is_signed_integer_dtype", "is_sparse", "is_string_dtype", "is_timedelta64_dtype", "is_timedelta64_ns_dtype", "is_unsigned_integer_dtype", "pandas_dtype"]



"""
This type stub file was generated by pyright.
"""

def ensure_decoded(s) -> str:
    """
    If we have bytes, decode them to unicode.
    """
    ...

def result_type_many(*arrays_and_dtypes): # -> dtype[Any]:
    """
    Wrapper around numpy.result_type which overcomes the NPY_MAXARGS (32)
    argument limit.
    """
    ...




"""
This type stub file was generated by pyright.
"""

import numpy as np

def fallback_performancewarning(version: str | None = ...) -> None:
    """
    Raise a PerformanceWarning for falling back to ExtensionArray's
    non-pyarrow method
    """
    ...

def pyarrow_array_to_numpy_and_mask(arr, dtype: np.dtype) -> tuple[np.ndarray, np.ndarray]:
    """
    Convert a primitive pyarrow.Array to a numpy array and boolean mask based
    on the buffers of the Array.

    At the moment pyarrow.BooleanArray is not supported.

    Parameters
    ----------
    arr : pyarrow.Array
    dtype : numpy.dtype

    Returns
    -------
    (data, mask)
        Tuple of two numpy arrays with the raw data (with specified dtype) and
        a boolean mask (validity mask, so False means missing)
    """
    ...




"""
This type stub file was generated by pyright.
"""

import string
import numpy as np

def randbool(size=..., p: float = ...): # -> bool:
    ...

RANDS_CHARS = np.array(list(string.ascii_letters + string.digits), dtype=(np.str_, 1))
RANDU_CHARS = np.array(list("".join(map(chr, range(1488, 1488 + 26))) + string.digits), dtype=(np.unicode_, 1))
def rands_array(nchars, size, dtype=..., replace=...) -> np.ndarray:
    """
    Generate an array of byte strings.
    """
    ...

def rands(nchars) -> str:
    """
    Generate one random byte string.

    See `rands_array` if you want to create an array of random strings.

    """
    ...

